//***************************************************************
//*  [[ A D V E R T E N C I A ]] =[ http://Solar-OpenSource.Org ]=
//*
//*  LICENCIA PARA LA DISTRIBUCIÓN DE WAR 3D
//*  1.- WAR 3D es un programa que podrá ser distribuido bajo los términos de la Licencia Pública General, además deberán cumplirse las siguientes condiciones.
//*  2.- Al copiar cualquier parte del código para ponerlo en un programa diferente a WAR 3D, el autor de dicho programa estará de acuerdo en que el código completo de su programa pasa a ser software libre, por lo deberá estar apegado a los términos establecidos por la licencia LGP. Es decir que, el código de WAR 3D solo puede ser tomado si se va a usar en otro proyecto de software libre. De lo contrario absténgase de usar el código o parte de el código de WAR3D.
//*  3.- Queda estrictamente prohibido modificar los nombres que aparecen en los créditos del programa, solo el grupo de desarrollo de  Solar-OpenSource podrá añadir nuevos nombres en los créditos. Si ha hecho alguna modificación y quiere que su nombre aparezca en los créditos debe ponerse en contacto con nosotros.
//*  4.- Cualquier modificación del código debe apegarse en lo posible a las normas que ha establecido el grupo de desarrollo de Solar-OpenSource, esto es para mayor claridad en el código. Dichas normas pueden encontrarse en nuestra pagina web http://www.solar-opensource.org
//*  5.- Puede copiar y distribuir el programa siempre y cuando no olvide: Acompañarlo con el código fuente completo correspondiente, legible para la máquina, así como el código escrito por usted o las modificaciones hechas al código original.
//*  6.- No debe distribuirse el ejecutable sin el código fuente.
//*  7.- Este texto no debe suprimirse. Las distribuciones deben acompañarse con la información que se recibió así como el ofrecimiento para distribuir el código fuente correspondiente.
//*
//***************************************************************

#include "ServidorDeDisparos.h"
#include "ServidorDeSprites.h"

// ---------------------------
// ** class TipoProyectil_Base ** //
// ---------------------------



// ---------------------------
// ** class Proyectil ** //
// ---------------------------

// ------------------------------------
Proyectil::Proyectil (z3DVector Inicio, z3DVector Fin, TipoProyectil_Base *MyProyectil)
{

  TrayectoriaFinalizada = false;

  this->Localizacion = Inicio;
  this->Fin = Fin;
  this->MyProyectil = MyProyectil;

  float Magnitud =  (Localizacion.x - Fin.x) * (Localizacion.x - Fin.x) +
    (Localizacion.y - Fin.y) * (Localizacion.y - Fin.y) +
    (Localizacion.z - Fin.z) * (Localizacion.z - Fin.z);
  Magnitud =  (float)pow((double)Magnitud, (double)0.5);


  Direccion.x = (Fin.x - Localizacion.x) / Magnitud;
  Direccion.y = (Fin.y - Localizacion.y) / Magnitud;
  Direccion.z = (Fin.z - Localizacion.z) / Magnitud;

}

// ------------------------------------
void Proyectil::setRetardo(float Retardo)
{

  Localizacion.x += MyProyectil->velocidad * Direccion.x * 1.0/Retardo;
  Localizacion.y += MyProyectil->velocidad * Direccion.y * 1.0/Retardo;
  Localizacion.z += MyProyectil->velocidad * Direccion.z * 1.0/Retardo;

  if ((Fin.x - Localizacion.x) * Direccion.x < 0)
    {
      TrayectoriaFinalizada = true;
    }

}


// ------------------------------------
void Proyectil::Dibujame()
{
  MyProyectil->Dibujame(Localizacion, Direccion);
}

// ------------------------------------
void Proyectil::onBlanco ()
{
  MyProyectil->onBlanco(Fin);	
}

// ---------------------------
// ** class ServidorDeDisparos ** //
// ---------------------------

// ------------------------------------


// ------------------------------------
ServidorDeDisparos::ServidorDeDisparos()
{
  
  TiposDeProyectil = new TipoProyectil_Base*[NumTiposDeProyectil];
  
  // Tipo BALA
  TiposDeProyectil[0] = new TipoProyectilBALA();
  TiposDeProyectil[1] = new TipoProyectilMISIL();
  TiposDeProyectil[2] = new TipoProyectilBOLA();

}

// ------------------------------------
void ServidorDeDisparos::Nuevo(z3DVector Inicio, z3DVector Fin, int Tipo)
{
  if (Tipo <= NumTiposDeProyectil && Tipo > 0)
    {
      Proyectil *NuevoProyectil = new Proyectil(Inicio, Fin, TiposDeProyectil[Tipo-1]);
      ListaDeProyectiles.push_back(NuevoProyectil);
    }
  else
    {
      cout << "Tipo de Disparo :" << Tipo << endl;
      War3DError("SERVIDORDEDISPAROS", "Tipo de protectil no soportado");
    }
}

// ------------------------------------
void ServidorDeDisparos::setRetardo(float Retardo)
{
  vector<Proyectil*>::iterator Ite;
  for (Ite = ListaDeProyectiles.begin() ; Ite != ListaDeProyectiles.end() ; Ite++)
    {
      (*Ite)->setRetardo(Retardo);
    }

  for (Ite = ListaDeProyectiles.begin() ; Ite != ListaDeProyectiles.end() ; Ite++)
    {
      if ((*Ite)->Desapareceme())
	{
	  (*Ite)->onBlanco ();
	  ListaDeProyectiles.erase(Ite);
	  return;
	}
    }

}

// ------------------------------------
void ServidorDeDisparos::Dibujame()
{
  vector<Proyectil*>::iterator Ite;
  for (Ite = ListaDeProyectiles.begin() ; Ite != ListaDeProyectiles.end() ; Ite++)
    {
      (*Ite)->Dibujame();
    }
}



//--------------------------------------------------------//
//--------------------------------------------------------//
//--------------------------------------------------------//
//--------------------------------------------------------//
//--------------------------------------------------------//
//--------------------------------------------------------//
//--------------------------------------------------------//


// ---------------------------
// ** class TipoProyectil_BALA ** //
// ---------------------------

// ------------------------------------
TipoProyectilBALA::TipoProyectilBALA()
{
  velocidad = 2.00;
}

TipoProyectilBALA::~TipoProyectilBALA()
{
  
}
// ------------------------------------
void TipoProyectilBALA::setRetardo(float Retardo)
{
}

// ------------------------------------
void TipoProyectilBALA:: Dibujame (z3DVector Localizacion, z3DVector Direccion)
{
  glPushMatrix();

  Direccion.x = Direccion.x * 0.13;
  Direccion.y = Direccion.y * 0.13;
  Direccion.z = Direccion.z * 0.13;

  glLineWidth(3.0); 

  glTranslatef(Localizacion.x, Localizacion.y, Localizacion.z);

  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
 

  glBegin(GL_LINES);
  {
    glColor4f(1.0, 1.0, 0.5 , 1.0);
    glVertex3f(0,0,0);
    
    glColor4f(1.0, 0.7, 0.7 , 0.0);
    glVertex3f(-Direccion.x, -Direccion.y, -Direccion.z);

  }
  glEnd();

  glPopMatrix();
}

// ------------------------------------
void TipoProyectilBALA::onBlanco (z3DVector Fin)
{
  ServidorDeSprites Explosion;
  Explosion.Nueva(Fin.x, Fin.y, Fin.z, 0, 0.01);
}


//--------------------------------------------------------//
//--------------------------------------------------------//


// ---------------------------
// ** class TipoProyectil_BOLA ** //
// ---------------------------

// ------------------------------------
TipoProyectilBOLA::TipoProyectilBOLA()
{
  velocidad = 3.00;
}

TipoProyectilBOLA::~TipoProyectilBOLA()
{

}

// ------------------------------------
void TipoProyectilBOLA::setRetardo(float Retardo)
{
}

// ------------------------------------
void TipoProyectilBOLA:: Dibujame (z3DVector Localizacion, z3DVector Direccion)
{
  glPushMatrix();

  Direccion.x = Direccion.x * 0.13;
  Direccion.y = Direccion.y * 0.13;
  Direccion.z = Direccion.z * 0.13;



  glTranslatef(Localizacion.x, Localizacion.y, Localizacion.z);

  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
 
  for (int i = 0 ; i < 5 ; i++)
    {

      glPointSize(5.0 - (i/2.0));
      
      glBegin(GL_POINTS);
      {
	glColor4f(0.7, 0.7, 1.0 , (5-i)/5.0);
	glVertex3f(-Direccion.x * (i*0.3), -Direccion.y * (i*0.3), -Direccion.z * (i*0.3));
      }
      glEnd();
    }

  glPopMatrix();
}

// ------------------------------------
void TipoProyectilBOLA::onBlanco (z3DVector Fin)
{
  ServidorDeSprites Explosion;
  Explosion.Nueva(Fin.x, Fin.y, Fin.z, 0, 0.03);
}




//--------------------------------------------------------//
//--------------------------------------------------------//


// ---------------------------
// ** class TipoProyectil_MISIL ** //
// ---------------------------

// ------------------------------------
TipoProyectilMISIL::TipoProyectilMISIL()
{
  velocidad = 4.00;
}

TipoProyectilMISIL::~TipoProyectilMISIL()
{
  
}

// ------------------------------------
void TipoProyectilMISIL::setRetardo(float Retardo)
{
}

// ------------------------------------
void TipoProyectilMISIL:: Dibujame (z3DVector Localizacion, z3DVector Direccion)
{
  glPushMatrix();

  Direccion.x = Direccion.x * 0.40;
  Direccion.y = Direccion.y * 0.40;
  Direccion.z = Direccion.z * 0.40;

  glLineWidth(5.0); 

  glTranslatef(Localizacion.x, Localizacion.y, Localizacion.z);

  glDisable(GL_LIGHTING);
  glDisable(GL_TEXTURE_2D);
 

  glBegin(GL_LINES);
  {
    glColor4f(1.0, 1.0, 0.8 , 7.0);
    glVertex3f(0,0,0);
    
    glColor4f(1.0, 1.0, 1.0 , 0.0);
    glVertex3f(-Direccion.x, -Direccion.y, -Direccion.z);

  }
  glEnd();

  glPopMatrix();

  ServidorDeSprites Explosion;
  Explosion.Nueva(Localizacion.x, Localizacion.y, Localizacion.z, 2, 0.010);
}

// ------------------------------------
void TipoProyectilMISIL::onBlanco (z3DVector Fin)
{
  ServidorDeSprites Explosion;
  Explosion.Nueva(Fin.x, Fin.y, Fin.z, 0, 0.04);
}
