#include "byKmath.h"

/* POINT2 --> */
POINT2::POINT2()
{
	x=0.0f;	y=0.0f;
}

POINT2::POINT2(float tx,float ty)
{
	x=tx;	y=ty;
}

const POINT2 POINT2::operator=(const POINT2 &p)
{
	x=p.x;	y=p.y;
	return *this;
}

const bool POINT2::operator==(const POINT2 &p)
{
	return ((x==p.x) && (y==p.y));
}

const bool POINT2::operator!=(const POINT2 &p)
{
	return !(*this==p);
}

const POINT2 POINT2::operator+(const POINT2 &p)
{
	return POINT2(x+p.x,y+p.y);
}

const POINT2 POINT2::operator+(void)
{
	return POINT2(*this);
}

const POINT2 POINT2::operator+=(const POINT2 &p)
{
	x+=p.x;	y+=p.y;
	return *this;
}

const POINT2 POINT2::operator-(const POINT2 &p)
{
	return POINT2(x-p.x,y-p.y);
}
     
const POINT2 POINT2::operator-()
{
	return POINT2(-x,-y);
}

const POINT2 POINT2::operator-=(const POINT2 &p)
{
	x-=p.x;	y-=p.y;
	return *this;
}

const POINT2 POINT2::operator*=(const float &s)
{
	x*=s;	y*=s;
	return *this;
}

const POINT2 POINT2::operator/=(const float &s)
{
	x/=s;	y/=s;
	return *this;
}

const POINT2 POINT2::operator*(const float &s)
{
	return POINT2(x*s,y*s);
}

const POINT2 POINT2::operator*(const POINT2 &p)
{
	return POINT2(x*p.x,y*p.y);
}

const POINT2 POINT2::operator/(const float s)
{	
	return POINT2(x/s,y/s);
}

const float& POINT2::operator [](int n)
{
	return *(float*)(this+n);
}

const float POINT2::Dot(POINT2 &p)
{	
	return x*p.x+y*p.y;
}

const float POINT2::Length(void)
{
	return (float)sqrt((float)(x*x+y*y));
}

const float POINT2::SquaredDistance(POINT2 &p)
{
	return (((x-p.x)*(x-p.x))+((y-p.y)*(y-p.y)));
}

const float POINT2::Distancia(float px,float py)
{
	//return byKraiz(SquaredDistance(VECTOR3D(px,py)),2);

	return byKraiz(byKpotencia(x-px,2)+byKpotencia(y-py,2),2);
}

const float POINT2::Distancia(const POINT2 &p)
{
	//return byKraiz(SquaredDistance(p),2);

	return byKraiz(byKpotencia(x-p.x,2)+byKpotencia(y-p.y,2),2);
}

const POINT2 POINT2::Unit(void)
{
	return (*this)/ Length();
}

const POINT2 POINT2::Transladar(float tx,float ty)
{
	return POINT2(x+tx,y+ty);
}

const POINT2 POINT2::RotarZ(float az)
{
	return POINT2(x*byKcos(az)+y*-byKsin(az),x*byKsin(az)+y*byKcos(az));
}

const POINT2 POINT2::RotarZAPos(float az,float px,float py)
{
	POINT2 rp,r;
	rp.SetProperties(x-px,y-py);
	r=rp.RotarZ(az);
	r=r.Transladar(x,y);
	return r;
}

const POINT2 POINT2::Scalar(float scx,float scy)
{
	return POINT2(x*scx,y*scy);
}

const POINT2 POINT2::Lerp(POINT2 &p,float t)
{
	POINT2 j=p-*this;
	return (*this+(j*t));
}

const void POINT2::Normalize(void)
{
	(*this)/=Length();
}

const void POINT2::SetProperties(float sx,float sy)
{
	x=sx;	y=sy;
}

POINT2::~POINT2()
{
}
/* <-- POINT2 */

/* POINT3 --> */
POINT3::POINT3()
{
	x=0.0f;	y=0.0f;	z=0.0f;
}

POINT3::POINT3(float tx,float ty,float tz)
{
	x=tx;	y=ty;	z=tz;
}

const POINT3 POINT3::operator=(const POINT3 &p)
{
	x=p.x;	y=p.y;	z=p.z;
	return *this;
}

const bool POINT3::operator==(const POINT3 &p)
{
	return ((x==p.x) && (y==p.y) && (z==p.z));
}

const bool POINT3::operator!=(const POINT3 &p)
{
	return !(*this==p);
}

const POINT3 POINT3::operator+(const POINT3 &p)
{
	return POINT3(x+p.x,y+p.y,z+p.z);
}

const POINT3 POINT3::operator+(void)
{
	return POINT3(*this);
}

const POINT3 POINT3::operator+=(const POINT3 &p)
{
	x+=p.x;	y+=p.y;	z+=p.z;
	return *this;
}

const POINT3 POINT3::operator-(const POINT3 &p)
{
	return POINT3(x-p.x,y-p.y,z-p.z);
}
     
const POINT3 POINT3::operator-()
{
	return POINT3(-x,-y,-z);
}

const POINT3 POINT3::operator-=(const POINT3 &p)
{
	x-=p.x;	y-=p.y;	z-=p.z;
	return *this;
}

const POINT3 POINT3::operator*=(const float &s)
{
	x*=s;	y*=s;	z*=s;     
	return *this;
}

const POINT3 POINT3::operator/=(const float &s)
{
	x/=s;	y/=s;	z/=s;
	return *this;
}

const POINT3 POINT3::operator*(const float &s)
{
	return POINT3(x*s,y*s,z*s);
}

const POINT3 POINT3::operator*(const POINT3 &p)
{
	return POINT3(x*p.x,y*p.y,z*p.z);
}

const POINT3 POINT3::operator/(const float s)
{
	return POINT3(x/s,y/s,z/s);
}

const float& POINT3::operator [](int n)
{
	return *(float*)(this+n);
}

const POINT3 POINT3::Cross(POINT3 &p)
{
	return POINT3(y*p.z-z*p.y,z*p.x-x*p.z,x*p.y-y*p.x);
}

const float POINT3::Dot(POINT3 &p)
{
	return x*p.x+y*p.y+z*p.z;
}

const float POINT3::Length(void)
{
	return (float)sqrt((float)(x*x+y*y+z*z));
}

const float POINT3::SquaredDistance(POINT3 &p)
{
	return (((x-p.x)*(x-p.x))+((y-p.y)*(y-p.y))+((z-p.z)*(z-p.z)));
}

const float POINT3::Distancia(float px,float py,float pz)
{
	//return byKraiz(SquaredDistance(VECTOR3D(px,py,pz)),2);
	return byKraiz(byKpotencia(x-px,2)+byKpotencia(y-py,2)+byKpotencia(z-pz,2),2);
}

const float POINT3::Distancia(const POINT3 &p)
{
	//return byKraiz(SquaredDistance(p),2);
	return byKraiz(byKpotencia(x-p.x,2)+byKpotencia(y-p.y,2)+byKpotencia(z-p.z,2),2);
}

const POINT3 POINT3::Unit(void)
{
	return (*this)/Length();
}

const POINT3 POINT3::Transladar(float tx,float ty,float tz)
{
	return POINT3(x+tx,y+ty,z+tz);
}

const POINT3 POINT3::RotarX(float ax)
{
	return POINT3(x,y*byKcos(ax)+z*-byKsin(ax),y*byKsin(ax)+z*byKcos(ax));
}

const POINT3 POINT3::RotarY(float ay)
{
	return POINT3(x*byKcos(ay)+z*byKsin(ay),y,x*-byKsin(ay)+z*byKcos(ay));
}

const POINT3 POINT3::RotarZ(float az)
{
	return POINT3(x*byKcos(az)+y*-byKsin(az),x*byKsin(az)+y*byKcos(az),z);
}

const POINT3 POINT3::RotarXAPos(float ax,float px,float py,float pz)
{
	POINT3 rp,r;
	rp.SetProperties(x-px,y-py,z-pz);
	r=rp.RotarX(ax);
	r=r.Transladar(x,y,z);
	return r;
}

const POINT3 POINT3::RotarYAPos(float ay,float px,float py,float pz)
{
	POINT3 rp,r;
	rp.SetProperties(x-px,y-py,z-pz);
	r=rp.RotarY(ay);
	r=r.Transladar(x,y,z);
	return r;
}

const POINT3 POINT3::RotarZAPos(float az,float px,float py,float pz)
{
	POINT3 rp,r;
	rp.SetProperties(x-px,y-py,z-pz);
	r=rp.RotarZ(az);
	r=r.Transladar(x,y,z);
	return r;
}

const POINT3 POINT3::Scalar(float scx,float scy,float scz)
{
	return POINT3(x*scx,y*scy,z*scz);
}

const POINT3 POINT3::Lerp(POINT3 &p,float t)
{
	POINT3 j=p-*this;
	return (*this+(j*t));
}

const void POINT3::Normalize(void)
{
	if(Length()){	(*this)/=Length();	} else {	SetProperties(0.0f,0.0f,0.0f);	}
}

const void POINT3::SetProperties(float sx,float sy,float sz)
{
	x=sx;	y=sy;	z=sz;
}

POINT3::~POINT3()
{
}
/* <-- POINT3 */

/* POINT4 --> */
POINT4::POINT4()
{
	x=0.0f;	y=0.0f;	z=0.0f;
}

POINT4::POINT4(float tx,float ty,float tz,float tw)
{
	x=tx;	y=ty;	z=tz;	w=tw;
}

const POINT4 POINT4::operator=(const POINT4 &p)
{
	x=p.x;	y=p.y;	z=p.z;	w=p.w;
	return *this;
}

const bool POINT4::operator==(const POINT4 &p)
{
	return ((x==p.x) && (y==p.y) && (z==p.z) && (w==p.w));
}

const bool POINT4::operator!=(const POINT4 &p)
{
	return !(*this==p);
}

const POINT4 POINT4::operator+(const POINT4 &p)
{
	return POINT4(x+p.x,y+p.y,z+p.z,w+p.w);
}

const POINT4 POINT4::operator+(void)
{
	return POINT4(*this);
}

const POINT4 POINT4::operator+=(const POINT4 &p)
{
	x+=p.x;	y+=p.y;	z+=p.z;	w+=p.w;
	return *this;
}

const POINT4 POINT4::operator-(const POINT4 &p)
{
	return POINT4(x-p.x,y-p.y,z-p.z,w-p.w);
}
     
const POINT4 POINT4::operator-()
{
	return POINT4(-x,-y,-z,-w);
}

const POINT4 POINT4::operator-=(const POINT4 &p)
{
	x-=p.x;	y-=p.y;	z-=p.z;	w-=p.w;
	return *this;
}

const POINT4 POINT4::operator*=(const float &s)
{
	x*=s;	y*=s;	z*=s;	w*=s;
	return *this;
}

const POINT4 POINT4::operator/=(const float &s)
{
	x/=s;	y/=s;	z/=s;	w/=s;
	return *this;
}

const POINT4 POINT4::operator*(const float &s)
{
	return POINT4(x*s,y*s,z*s,w*s);
}

const POINT4 POINT4::operator*(const POINT4 &p)
{
	return POINT4(x*p.x,y*p.y,z*p.z,w*p.z);
}

const POINT4 POINT4::operator/(const float s)
{
	return POINT4(x/s,y/s,z/s,w/s);
}

const float& POINT4::operator [](int n)
{
	return *(float*)(this+n);
}

const POINT4 POINT4::Cross(POINT4 &p)
{
	//return POINT4(y*p.z-z*p.y,z*p.x-x*p.z,x*p.y-y*p.x);	<-- VeR
	return *this;
}

const float POINT4::Dot(POINT4 &p)
{	
	return x*p.x+y*p.y+z*p.z+w*p.w;
}

const float POINT4::Length(void)
{
	return (float)sqrt((float)(x*x+y*y+z*z+w*w));
}

const float POINT4::SquaredDistance(POINT4 &p)
{
	return (((x-p.x)*(x-p.x))+((y-p.y)*(y-p.y))+((z-p.z)*(z-p.z))+((w-p.w)*(w-p.w)));
}

const float POINT4::Distancia(float px,float py,float pz,float pw)
{
	//return byKraiz((float)SquaredDistance(*this),2); MAL
	return byKraiz(byKpotencia(x-px,2)+byKpotencia(y-py,2)+byKpotencia(z-pz,2)+byKpotencia(w-pw,2),2);
}

const float POINT4::Distancia(const POINT4 &p)
{
	//return byKraiz(SquaredDistance(p),2);
	return byKraiz(byKpotencia(x-p.x,2)+byKpotencia(y-p.y,2)+byKpotencia(z-p.z,2)+byKpotencia(w-p.w,2),2);
}

const POINT4 POINT4::Unit(void)
{
	return (*this)/ Length();
}

const POINT4 POINT4::Transladar(float tx,float ty,float tz,float tw)
{
	return POINT4(x+tx,y+ty,z+tz,w+tw);
}

const POINT4 POINT4::RotarX(float ax)
{
	return POINT4(x,y*byKcos(ax)+z*-byKsin(ax),y*byKsin(ax)+z*byKcos(ax),w);
}

const POINT4 POINT4::RotarY(float ay)
{
	return POINT4(x*byKcos(ay)+z*byKsin(ay),y,x*-byKsin(ay)+z*byKcos(ay),w);
}

const POINT4 POINT4::RotarZ(float az)
{
	return POINT4(x*byKcos(az)+y*-byKsin(az),x*byKsin(az)+y*byKcos(az),z,w);
}

const POINT4 POINT4::RotarXAPos(float ax,float px,float py,float pz)
{
	POINT4 rp,r;
	rp.SetProperties(x-px,y-py,z-pz,w);
	r=rp.RotarX(ax);
	r=r.Transladar(x,y,z,w);
	return r;
}

const POINT4 POINT4::RotarYAPos(float ay,float px,float py,float pz)
{
	POINT4 rp,r;
	rp.SetProperties(x-px,y-py,z-pz,w);
	r=rp.RotarY(ay);
	r=r.Transladar(x,y,z,w);
	return r;
}

const POINT4 POINT4::RotarZAPos(float az,float px,float py,float pz)
{
	POINT4 rp,r;
	rp.SetProperties(x-px,y-py,z-pz,w);
	r=rp.RotarZ(az);
	r=r.Transladar(x,y,z,w);
	return r;
}

const POINT4 POINT4::Scalar(float scx,float scy,float scz,float scw)
{
	return POINT4(x*scx,y*scy,z*scz,w*scw);
}

const POINT4 POINT4::Lerp(POINT4 &p,float t)
{
	POINT4 j=(p-*this);
	return (*this+(j*t));
}

const void POINT4::Normalize(void)
{
	(*this)/=Length();
}

const void POINT4::SetProperties(float sx,float sy,float sz,float sw)
{
	x=sx;	y=sy;	z=sz;	w=sw;
}

POINT4::~POINT4()
{
}
/* <-- POINT4 */

/* RECT2 --> */
RECT2::RECT2()
{
	m=0.0f;	e=0.0f;
}

RECT2::RECT2(float x1,float y1,float x2,float y2)
{
	m=(y2-y1)/(x2-x1);	e=m*(-x1)+y1;
}

RECT2::RECT2(POINT2 p1,POINT2 p2)
{
	m=(p2.y-p1.y)/(p2.x-p1.x);	e=m*(-p1.x)+p1.y;
}

RECT2::RECT2(float tm,float te)
{
	m=tm;	e=te;
}

const RECT2 RECT2::operator=(const RECT2 &r)
{
	m=r.m;	e=r.e;
	return *this;
}

const bool RECT2::operator==(const RECT2 &r)
{
	return (e==r.e && m==r.m);
}

const bool RECT2::operator!=(const RECT2 &r)
{
	return !(e==r.e && m==r.m);
}

const bool RECT2::paralelas(const RECT2 &r)
{
	return (e==r.e);
}

const bool RECT2::perpendiculares(const RECT2 &r)
{
	return !(paralelas(r));
}

const POINT2 RECT2::intersectar(const RECT2 &r)
{
	return POINT2((r.e-e)/(m-r.m),m*(r.e-e)/(m-r.m)+e);
}

const POINT2 RECT2::iejes(void)
{
	return POINT2(-e/m,e);
}

const float RECT2::calcularXY(const float Y)
{
	return ((Y-e)/m);
}

const float RECT2::calcularYX(const float X)
{
	return (m*X+e);
}

const float RECT2::distancia(const RECT2 &r)
{
	if(paralelas(r))
	{
		RECT2 q(-1/m,0);	POINT2 r1,r2;
		r1=q.intersectar(*this);	r2=q.intersectar(r);
		return r1.Distancia(r2);
	}
	return 0.0f;
}

const float RECT2::distancia(const POINT2 &p)
{
	RECT2 q(-1/m,-m*p.x+m*p.y);
	POINT2 r1=intersectar(q);
	return r1.Distancia(p);
}

const void RECT2::SetProperties(float sm,float se)
{
	m=sm;	e=se;
}

RECT2::~RECT2()
{
}
/* <-- RECT2 */

/* RECT3 --> */
RECT3::RECT3()
{
}

RECT3::RECT3(float x1,float y1,float z1,float x2,float y2,float z2)
{
	my=(y2-y1)/(x2-x1);	ey=my*(-x1)+y1;
	mz=(z2-z1)/(x2-x1);	ez=mz*(-x1)+z1;
}

RECT3::RECT3(POINT3 p1,POINT3 p2)
{
	my=(p2.y-p1.y)/(p2.x-p1.x);	ey=my*(-p1.x)+p1.y;
	mz=(p2.z-p1.z)/(p2.x-p1.x);	ez=mz*(-p1.x)+p1.z;
}

RECT3::RECT3(float tmy,float tey,float tmz,float tez)
{
	my=tmy;	ey=tey;	mz=tmz;	ez=tez;
}

const RECT3 RECT3::operator= (const RECT3 &r)
{
	my=r.my;	ey=r.ey;	mz=r.mz;	ez=r.ez;
	return *this;
}

const bool RECT3::operator== (const RECT3 &r)
{
	return (my==r.my && ey==r.ey && mz==r.mz && ez==r.ez);
}

const bool RECT3::operator!= (const RECT3 &r)
{
	return !(my==r.my && ey==r.ey && mz==r.mz && ez==r.ez);
}

const bool RECT3::paralelas (const RECT3 &r)
{
	//Is-This-True?
	return (ey==r.ey && ez==r.ez);
}

const bool RECT3::perpendiculares (const RECT3 &r)
{
	return !(paralelas(r));
}

const POINT3 RECT3::intersectar (const RECT3 &r)
{
	if((r.ey-ey)/(my-r.my)!=(r.ez-ez)/(mz-r.mz)){	return POINT3(-911,-911,-911);	}
	return POINT3((r.ey-ey)/(my-r.my),my*(r.ey-ey)/(my-r.my)+ey,mz*(r.ez-ez)/(mz-r.mz)+ez);
}

const POINT3 RECT3::iejes (void)
{
	return POINT3(-ey/my,ey,ez);
}

const float RECT3::calcularXY (const float Y)
{
	return (Y-ey)/my;
}

const float RECT3::calcularXZ (const float Z)
{
	return (Z-ez)/mz;
}

const float RECT3::calcularYX (const float X)
{
	return my*X+ey;
}

const float RECT3::calcularYZ (const float Z)
{
	return my*((Z-ez)/mz)+ey;
}

const float RECT3::calcularZX (const float X)
{
	return mz*X+ez;
}

const float RECT3::calcularZY (const float Y)
{
	return mz*((Y-ey)/my)+ez;
}

const float RECT3::distancia (const RECT3 &r)
{
//~	if(paralelas(r))
//~	{
//~		RECT3 q(-1/m,0);	POINT3 r1,r2;
//~		r1=q.intersectar(*this);	r2=q.intersectar(r);
//~		return r1.Distancia(r2);
//~	}
	return 0.0f;
}

const float RECT3::distancia (const POINT3 &p)
{
//~	RECT3 q(-1/m,-m*p.x+m*p.y);
//~	POINT3 r1=intersectar(q);
//~	return r1.Distancia(p);
	return 0.0f; //err, not this...
}

const void RECT3::SetProperties(float smy,float sey,float smz,float sez)
{
	my=smy;	ey=sey;	mz=smz;	ez=sez;
}

RECT3::~RECT3()
{
}
/* <-- RECT3 */

/* PLANE --> */
PLANE::PLANE()
{
}

PLANE::PLANE(POINT3 n,float td)
{
	a=n.x;	b=n.y;	c=n.z;	d=td;
}

PLANE::PLANE(POINT3 p,POINT3 q,POINT3 r)
{
	POINT3 v(q-p);	POINT3 w(r-p);	POINT3 n(v.Cross(w));	n.Normalize();
	a=n.x;	b=n.y;	c=n.z;	d=-n.Dot(p);
}

PLANE::PLANE(float ta,float tb,float tc,float td)
{
	a=ta;	b=tb;	c=tc;	d=td;
}

const PLANE PLANE::operator= (const PLANE &p)
{
	a=p.a;	b=p.b;	c=p.c;	d=p.d;
	return *this;
}

const bool PLANE::operator== (const PLANE &p)
{
	return ((a==p.a) && (b==p.b) && (c==p.c) && (d==p.d));
}

const bool PLANE::operator!= (const PLANE &p)
{
	return !(*this==p);
}

const POINT3 PLANE::intersectRect(RECT3 &r)
{
	POINT3 ip;
	ip.x=(-d-b*r.ey-c*r.ez)/(a+b*r.my+c*r.mz);
	ip.y=r.my*ip.x+r.ey;
	ip.z=r.mz*ip.x+r.ez;

	return ip;
}

const char PLANE::intersectTriangle(const POINT3 &p1,const POINT3 &p2,const POINT3 &p3)
{
	float r1=p1.x*a+p1.y*b+p1.z*c+d,r2=p2.x*a+p2.y*b+p2.z*c+d,r3=p3.x*a+p3.y*b+p3.z*c+d;
	
	if(r1<0 && r2<0 && r3<0)
	{
		return 0;
	}
	if(r1>0 && r2>0 && r3>0)
	{
		return +1;
	}
	return 0;
}

const float& PLANE::operator[] (int n)
{
	return *(float*)(this+n);
}

const void PLANE::SetProperties(POINT3 sn,float sd)
{
	a=sn.x;	b=sn.y;	c=sn.z;	d=sd;
}

const void PLANE::SetProperties(POINT3 sp,POINT3 sq,POINT3 sr)
{
	POINT3 v(sq-sp);	POINT3 w(sr-sp);	POINT3 n(v.Cross(w));	n.Normalize();
	a=n.x;	b=n.y;	c=n.z;	d=-n.Dot(sp);
}

const void PLANE::SetProperties(float sa,float sb,float sc,float sd)
{
	a=sa;	b=sb;	c=sc;	d=sd;
}

PLANE::~PLANE()
{
}
/* <-- PLANE */

/* QUATERNION --> */
QUATERNION::QUATERNION()
{
	x=0.0f;	y=0.0f;	z=0.0f;	w=1.0f;
}

QUATERNION::QUATERNION(float tx,float ty,float tz,float tw)
{
	x=tx;	y=ty;	z=tz;	w=tw;
}

const QUATERNION QUATERNION::operator=(const QUATERNION &qt)
{
	x=qt.x;	y=qt.y;	z=qt.z;	w=qt.w;
	return *this;
}

const QUATERNION QUATERNION::operator+(const QUATERNION &qt)
{
	return QUATERNION(x+qt.x,y+qt.y,z+qt.z,w+qt.w);
}

const QUATERNION QUATERNION::operator+(void)
{
	return *this;
}

const QUATERNION QUATERNION::operator+=(const QUATERNION &qt)
{
	x+=qt.x;	y+=qt.y;	z+=qt.z;	w+=qt.w;
	return *this;
}

const QUATERNION QUATERNION::operator-(const QUATERNION &qt)
{
	return QUATERNION(x-qt.x,y-qt.y,z-qt.z,w-qt.w);
}

const QUATERNION QUATERNION::operator-(void)
{
	return QUATERNION(-x,-y,-z,-w);
}

const QUATERNION QUATERNION::operator-=(const QUATERNION &qt)
{
	x-=qt.x;	y-=qt.y;	z-=qt.z;	w-=qt.w;
	return *this;
}

const QUATERNION QUATERNION::operator*(const QUATERNION &qt)
{
	return QUATERNION(w*qt.x+x*qt.w+y*qt.z-z*qt.y,w*qt.y-x*qt.z+y*qt.w+z*qt.x,w*qt.z+x*qt.y-y*qt.x+z*qt.w,w*qt.w-x*qt.x-y*qt.y-z*qt.z);
}

const QUATERNION QUATERNION::operator*(const float s)
{
	return QUATERNION(x*s,y*s,z*s,w*s);
}

const QUATERNION QUATERNION::operator*=(const QUATERNION &qt)
{
	QUATERNION qIt;
	qIt.x=w*qt.x+x*qt.w+y*qt.z-z*qt.y;
	qIt.y=w*qt.y-x*qt.z+y*qt.w+z*qt.x;	
	qIt.z=w*qt.z+x*qt.y-y*qt.x+z*qt.w;
	qIt.w=w*qt.w-x*qt.x-y*qt.y-z*qt.z;
	*this=qIt;
	return *this;
}

const QUATERNION QUATERNION::operator*=(const float s)
{
	x*=s;	y*=s;	z*=s;	w*=s;
	return *this;
}

const QUATERNION QUATERNION::operator/(const float s)
{
	return QUATERNION(x/s,y/s,z/s,w/s);
}

const QUATERNION QUATERNION::operator/=(const float s)
{
	x/=s;	y/=s;	z/=s;	w/=s;
	return *this;
}

const bool QUATERNION::operator==(const QUATERNION &qt)
{
	if(x==qt.x && y==qt.y && z==qt.z && w==qt.w){	return true;	}
	return false;
}

const bool QUATERNION::operator!=(const QUATERNION &qt)
{
	if(x==qt.x && y==qt.y && z==qt.z && w==qt.w){	return false;	}
	return true;
}

const float& QUATERNION::operator [](int n)
{
	return *(float*)(this+n);
}

const float QUATERNION::Dot(QUATERNION &qt)
{
	return x*qt.x+y*qt.y+z*qt.z+w*qt.w;
}

const float QUATERNION::Length(void)
{
	return (float)sqrt((float)Dot(*this));
}

const QUATERNION QUATERNION::Normal(void)
{
	float len=Length();
	return QUATERNION(x/len,y/len,z/len,w/len);
}

const void QUATERNION::CrearMatrix(MATRIX4X4 &mat)
{	
	mat.m[0][0]=1.0f-2.0f*(y*y+z*z);	mat.m[0][1]=2.0f*(x*y-w*z);	mat.m[0][2]=2.0f*(x*z+w*y);	mat.m[0][3]=0.0f;
	mat.m[1][0]=2.0f*(x*y+w*z);	mat.m[1][1]=1.0f-2.0f*(x*x+z*z);	mat.m[1][2]=2.0f*(y*z-w*x);	mat.m[1][3]=0.0f;
	mat.m[2][0]=2.0f*(x*z-w*y);	mat.m[2][1]=2.0f*(y*z+w*x);	mat.m[2][2]=1.0f-2.0f*(x*x+y*y);	mat.m[2][3]=0.0f;
	mat.m[3][0]=0.0f;	mat.m[3][1]=0.0f;	mat.m[3][2]=0.0f;	mat.m[3][3]=1.0f;
}

const void QUATERNION::CrearFromMatrix(MATRIX3X3 &mat)
{
	float diagonal=mat.m[0][0]+mat.m[1][1]+mat.m[2][2]+1,scale=0.0f;
	if(diagonal>0.00000001f)
	{
		scale=float(sqrt(diagonal)*2);
		x=(mat.m[2][1]-mat.m[1][2])/scale;	y=(mat.m[0][2]-mat.m[2][0])/scale;	z=(mat.m[1][0]-mat.m[0][1])/scale;	w=0.25f*scale;
	} else {
		if(mat.m[0][0]>mat.m[1][1] && mat.m[0][0]>mat.m[2][2])
		{
			scale=(float)sqrt(1.0f+mat.m[0][0]-mat.m[1][1]-mat.m[2][2])*2.0f;
			x=0.25f*scale;	y=(mat.m[1][0]+mat.m[0][1])/scale;	z=(mat.m[0][2]+mat.m[2][0])/scale;	w=(mat.m[2][1]-mat.m[1][2])/scale;
		} else if(mat.m[1][1]>mat.m[2][2]) {
			scale=(float)sqrt(1.0f+mat.m[1][1]-mat.m[0][0]-mat.m[2][2])*2.0f;
			x=(mat.m[1][0]+mat.m[0][1])/scale;	y=0.25f*scale;	z=(mat.m[2][1]+mat.m[1][2])/scale;	w=(mat.m[0][2]-mat.m[2][0])/scale;
		} else {	
			scale=(float)sqrt(1.0f+mat.m[2][2]-mat.m[0][0]-mat.m[1][1])*2.0f;
			x=(mat.m[0][2]+mat.m[2][0])/scale;	y=(mat.m[2][1]+mat.m[1][2])/scale;	z=0.25f*scale;	w=(mat.m[1][0]-mat.m[0][1])/scale;
		}
	}
}

const void QUATERNION::CrearFromMatrix(MATRIX4X4 &mat)
{
	float diagonal=mat.m[0][0]+mat.m[1][1]+mat.m[2][2]+1,scale=0.0f;
	if(diagonal>0.00000001f)
	{
		scale=float(sqrt(diagonal)*2);
		x=(mat.m[2][1]-mat.m[1][2])/scale;	y=(mat.m[0][2]-mat.m[2][0])/scale;	z=(mat.m[1][0]-mat.m[0][1])/scale;	w=0.25f*scale;
	} else {
		if(mat.m[0][0]>mat.m[1][1] && mat.m[0][0]>mat.m[2][2])
		{
			scale=(float)sqrt(1.0f+mat.m[0][0]-mat.m[1][1]-mat.m[2][2])*2.0f;
			x=0.25f*scale;	y=(mat.m[1][0]+mat.m[0][1])/scale;	z=(mat.m[0][2]+mat.m[2][0])/scale;	w=(mat.m[2][1]-mat.m[1][2])/scale;
		} else if(mat.m[1][1]>mat.m[2][2]) {
			scale=(float)sqrt(1.0f+mat.m[1][1]-mat.m[0][0]-mat.m[2][2])*2.0f;
			x=(mat.m[1][0]+mat.m[0][1])/scale;	y=0.25f*scale;	z=(mat.m[2][1]+mat.m[1][2])/scale;	w=(mat.m[0][2]-mat.m[2][0])/scale;
		} else {	
			scale=(float)sqrt(1.0f+mat.m[2][2]-mat.m[0][0]-mat.m[1][1])*2.0f;
			x=(mat.m[0][2]+mat.m[2][0])/scale;	y=(mat.m[2][1]+mat.m[1][2])/scale;	z=0.25f*scale;	w=(mat.m[1][0]-mat.m[0][1])/scale;
		}
	}
}

const QUATERNION QUATERNION::Slerp(QUATERNION &q1,QUATERNION &q2,float t)
{
	QUATERNION qIt;

	float result=(q1.x*q2.x)+(q1.y*q2.y)+(q1.z*q2.z)+(q1.w*q2.w);
	if(result<0.0f){	q2=QUATERNION(-q2.x,-q2.y,-q2.z,-q2.w);	result=-result;	}
	float scale0=1-t,scale1=t;
	if(1-result>0.1f)
	{
		float theta=(float)acos(result);	float sinTheta=(float)sin(theta);
		scale0=(float)sin((1-t)*theta)/sinTheta;	scale1=(float)sin((t*theta))/sinTheta;
	}
	qIt.x=(scale0*q1.x)+(scale1*q2.x);	qIt.y=(scale0*q1.y)+(scale1*q2.y);	qIt.z=(scale0*q1.z)+(scale1*q2.z);	qIt.w=(scale0*q1.w)+(scale1*q2.w);
	return qIt;
}

const QUATERNION QUATERNION::Lerp(QUATERNION &q1,QUATERNION &q2,float t)
{
	QUATERNION qIt;
	qIt.x=(q1.x+((q2.x-q1.x)*t));	qIt.y=(q1.y+((q2.y-q1.y)*t));	qIt.z=(q1.z+((q2.z-q1.z)*t));	qIt.w=(q1.w+((q2.w-q1.w)*t));
	return qIt;
}

const void QUATERNION::Normalize(void)
{
	(*this)/=Length();
}

const void QUATERNION::SetProperties(float sx,float sy,float sz,float sw)
{
	x=sx;	y=sy;	z=sz;	w=sw;
}

QUATERNION::~QUATERNION()
{
}
/* <-- QUATERNION */

/* MATRIX2X2 --> */
MATRIX2X2::MATRIX2X2()
{
}

MATRIX2X2::MATRIX2X2(float _a,float _b,float _c,float _d)
{
	m[0][0]=_a;	m[0][1]=_b;
	m[1][0]=_c;	m[1][1]=_d;
}

const MATRIX2X2 MATRIX2X2::operator=(const MATRIX2X2 &mt)
{
	m[0][0]=mt.m[0][0];	m[0][1]=mt.m[0][1];
	m[1][0]=mt.m[1][0];	m[1][1]=mt.m[1][1];
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator+(const MATRIX2X2 &mt)
{
	return MATRIX2X2(
	m[0][0]+mt.m[0][0],m[0][1]+mt.m[0][1],
	m[1][0]+mt.m[1][0],m[1][1]+mt.m[1][1]);
}

const MATRIX2X2 MATRIX2X2::operator+(void)
{
	return *this;
};

const MATRIX2X2 MATRIX2X2::operator+=(const MATRIX2X2 &mt)
{
	m[0][0]+=mt.m[0][0];	m[0][1]+=mt.m[0][1];
	m[1][0]+=mt.m[1][0];	m[1][1]+=mt.m[1][1];
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator-(const MATRIX2X2 &mt)
{
	return MATRIX2X2(
	m[0][0]-mt.m[0][0],m[0][1]-mt.m[0][1],
	m[1][0]-mt.m[1][0],m[1][1]-mt.m[1][1]);
}

const MATRIX2X2 MATRIX2X2::operator-(void)
{
	return MATRIX2X2(
	-m[0][0],-m[0][1],
	-m[1][0],-m[1][1]);
}

const MATRIX2X2 MATRIX2X2::operator-=(const MATRIX2X2 &mt)
{
	m[0][0]-=mt.m[0][0];	m[0][1]-=mt.m[0][1];
	m[1][0]-=mt.m[1][0];	m[1][1]-=mt.m[1][1];
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator*(const MATRIX2X2 &mt)
{
	return MATRIX2X2(
	m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0],
	m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1],
	m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0],
	m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1]);
}

const MATRIX2X2 MATRIX2X2::operator*(const float s)
{
	return MATRIX2X2(
	m[0][0]*s,m[0][1]*s,
	m[1][0]*s,m[1][1]*s);
}

const MATRIX2X2 MATRIX2X2::operator*=(const MATRIX2X2 &mt)
{
	m[0][0]=m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0];
	m[0][1]=m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1];
	m[1][0]=m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0];
	m[1][1]=m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1];
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator*=(const float s)
{
	m[0][0]*=s;	m[0][1]*=s;
	m[1][0]*=s;	m[1][1]*=s;
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator/(const MATRIX2X2 &mt)
{
//~	return *this*mt.Inverse();
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator/(const float s)
{
	return MATRIX2X2(
	m[0][0]/s,m[0][1]/s,
	m[1][0]/s,m[1][1]/s);
}

const MATRIX2X2 MATRIX2X2::operator/=(const MATRIX2X2 &mt)
{
//~	*this*=mt.Inverse();
	return *this;
}

const MATRIX2X2 MATRIX2X2::operator/=(const float s)
{
	m[0][0]/=s;	m[0][1]/=s;
	m[1][0]/=s;	m[1][1]/=s;
	return *this;
}

const bool MATRIX2X2::operator==(const MATRIX2X2 &mt)
{
	return (
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]));
}

const bool MATRIX2X2::operator!=(const MATRIX2X2 &mt)
{
	return !(
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]));
}

const MATRIX2X2 MATRIX2X2::Inverse(void)
{
	MATRIX2X2 r(1,0,0,1);
	float _m[2][2],tp;	memcpy(_m,m,2*2*sizeof(float));

	tp=_m[0][0];	_m[0][0]/=tp;			_m[0][1]/=tp;			r.m[0][0]/=tp;
	tp=_m[1][0];	_m[1][0]-=tp*_m[0][0];	_m[1][1]-=tp*_m[0][1];	r.m[1][0]-=tp*r.m[0][0];

	tp=_m[1][1];	_m[1][1]/=tp;			r.m[1][0]/=tp;				r.m[1][1]/=tp;
	tp=_m[0][1];	_m[0][1]-=tp*_m[1][1];	r.m[0][0]-=tp*r.m[1][0];	r.m[0][1]-=tp*r.m[1][1];

	return r;
}

const MATRIX2X2 MATRIX2X2::Transpose(void)
{
	MATRIX2X2 r=*this;

	r.m[0][0]=m[0][0];	r.m[0][1]=m[1][0];
	r.m[1][0]=m[0][1];	r.m[1][1]=m[1][1];

	return r;
}

const void MATRIX2X2::SetProperties(float _a,float _b,float _c,float _d)
{
	m[0][0]=_a;	m[0][1]=_b;
	m[1][0]=_c;	m[1][1]=_d;
}

MATRIX2X2::~MATRIX2X2() 
{
}
/* <-- MATRIX2X2 */

/* MATRIX3X3 --> */
MATRIX3X3::MATRIX3X3()
{
}

MATRIX3X3::MATRIX3X3(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i)
{
	m[0][0]=_a;	m[0][1]=_b;	m[0][2]=_c;
	m[1][0]=_d;	m[1][1]=_e;	m[1][2]=_f;
	m[2][0]=_g;	m[2][1]=_h;	m[2][2]=_i;
}

const MATRIX3X3 MATRIX3X3::operator=(const MATRIX3X3 &mt)
{
	m[0][0]=mt.m[0][0];	m[0][1]=mt.m[0][1];	m[0][2]=mt.m[0][2];
	m[1][0]=mt.m[1][0];	m[1][1]=mt.m[1][1];	m[0][2]=mt.m[1][2];
	m[2][0]=mt.m[2][0];	m[2][1]=mt.m[2][1];	m[0][2]=mt.m[2][2];
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator+(const MATRIX3X3 &mt)
{
	return MATRIX3X3(
	m[0][0]+mt.m[0][0],m[0][1]+mt.m[0][1],m[0][2]+mt.m[0][2],
	m[1][0]+mt.m[1][0],m[1][1]+mt.m[1][1],m[1][2]+mt.m[1][2],
	m[2][0]+mt.m[2][0],m[2][1]+mt.m[2][1],m[2][2]+mt.m[2][2]);
}

const MATRIX3X3 MATRIX3X3::operator+(void)
{
	return *this;
};

const MATRIX3X3 MATRIX3X3::operator+=(const MATRIX3X3 &mt)
{
	m[0][0]+=mt.m[0][0];	m[0][1]+=mt.m[0][1];	m[0][2]+=mt.m[0][2];
	m[1][0]+=mt.m[1][0];	m[1][1]+=mt.m[1][1];	m[0][2]+=mt.m[1][2];
	m[2][0]+=mt.m[2][0];	m[2][1]+=mt.m[2][1];	m[0][2]+=mt.m[2][2];
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator-(const MATRIX3X3 &mt)
{
	return MATRIX3X3(
	m[0][0]-mt.m[0][0],m[0][1]-mt.m[0][1],m[0][2]-mt.m[0][2],
	m[1][0]-mt.m[1][0],m[1][1]-mt.m[1][1],m[0][2]-mt.m[1][2],
	m[2][0]-mt.m[2][0],m[2][1]-mt.m[2][1],m[0][2]-mt.m[2][2]);
}

const MATRIX3X3 MATRIX3X3::operator-(void)
{
	return MATRIX3X3(
	-m[0][0],-m[0][1],-m[0][2],
	-m[1][0],-m[1][1],-m[0][2],
	-m[2][0],-m[2][1],-m[0][2]);
}

const MATRIX3X3 MATRIX3X3::operator-=(const MATRIX3X3 &mt)
{
	m[0][0]-=mt.m[0][0];	m[0][1]-=mt.m[0][1];	m[0][2]-=mt.m[0][2];
	m[1][0]-=mt.m[1][0];	m[1][1]-=mt.m[1][1];	m[0][2]-=mt.m[1][2];
	m[2][0]-=mt.m[2][0];	m[2][1]-=mt.m[2][1];	m[0][2]-=mt.m[2][2];
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator*(const MATRIX3X3 &mt)
{
	return MATRIX3X3(
	m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0]+m[0][2]*mt.m[2][0],
	m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1]+m[0][2]*mt.m[2][1],
	m[0][0]*mt.m[0][2]+m[0][1]*mt.m[1][2]+m[0][2]*mt.m[2][2],
	m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0]+m[1][2]*mt.m[2][0],
	m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1]+m[1][2]*mt.m[2][1],
	m[1][0]*mt.m[0][2]+m[1][1]*mt.m[1][2]+m[1][2]*mt.m[2][2],
	m[2][0]*mt.m[0][0]+m[2][1]*mt.m[1][0]+m[2][2]*mt.m[2][0],
	m[2][0]*mt.m[0][1]+m[2][1]*mt.m[1][1]+m[2][2]*mt.m[2][1],
	m[2][0]*mt.m[0][2]+m[2][1]*mt.m[1][2]+m[2][2]*mt.m[2][2]);
}

const MATRIX3X3 MATRIX3X3::operator*(const float s)
{
	return MATRIX3X3(
	m[0][0]*s,m[0][1]*s,m[0][2]*s,
	m[1][0]*s,m[1][1]*s,m[1][2]*s,
	m[2][0]*s,m[2][1]*s,m[2][2]*s);
}

const MATRIX3X3 MATRIX3X3::operator*=(const MATRIX3X3 &mt)
{
	m[0][0]=m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0]+m[0][2]*mt.m[2][0];
	m[0][1]=m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1]+m[0][2]*mt.m[2][1];
	m[0][2]=m[0][0]*mt.m[0][2]+m[0][1]*mt.m[1][2]+m[0][2]*mt.m[2][2];
	m[1][0]=m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0]+m[1][2]*mt.m[2][0];
	m[1][1]=m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1]+m[1][2]*mt.m[2][1];
	m[1][2]=m[1][0]*mt.m[0][2]+m[1][1]*mt.m[1][2]+m[1][2]*mt.m[2][2];
	m[2][0]=m[2][0]*mt.m[0][0]+m[2][1]*mt.m[1][0]+m[2][2]*mt.m[2][0];
	m[2][1]=m[2][0]*mt.m[0][1]+m[2][1]*mt.m[1][1]+m[2][2]*mt.m[2][1];
	m[2][2]=m[2][0]*mt.m[0][2]+m[2][1]*mt.m[1][2]+m[2][2]*mt.m[2][2];
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator*=(const float s)
{
	m[0][0]*=s;	m[0][1]*=s;	m[0][2]*=s;
	m[1][0]*=s;	m[1][1]*=s;	m[1][2]*=s;
	m[2][0]*=s;	m[2][1]*=s;	m[2][2]*=s;
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator/(const MATRIX3X3 &mt)
{
//~	return *this*mt.Inverse();
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator/(const float s)
{
	return MATRIX3X3(
	m[0][0]/s,m[0][1]/s,m[0][2]/s,
	m[1][0]/s,m[1][1]/s,m[1][2]/s,
	m[2][0]/s,m[2][1]/s,m[2][2]/s);
}

const MATRIX3X3 MATRIX3X3::operator/=(const MATRIX3X3 &mt)
{
//~	*this*=mt.Inverse();
	return *this;
}

const MATRIX3X3 MATRIX3X3::operator/=(const float s)
{
	m[0][0]/=s;	m[0][1]/=s;	m[0][2]/=s;
	m[1][0]/=s;	m[1][1]/=s;	m[1][2]/=s;
	m[2][0]/=s;	m[2][1]/=s;	m[2][2]/=s;
	return *this;
}

const bool MATRIX3X3::operator==(const MATRIX3X3 &mt)
{
	return (
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) && (m[0][2]==mt.m[0][2]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]) && (m[1][2]==mt.m[1][2]) &&
	(m[2][0]==mt.m[2][0]) && (m[2][1]==mt.m[2][1]) && (m[2][2]==mt.m[2][2]));
}

const bool MATRIX3X3::operator!=(const MATRIX3X3 &mt)
{
	return !(
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) && (m[0][2]==mt.m[0][2]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]) && (m[0][2]==mt.m[1][2]) &&
	(m[2][0]==mt.m[2][0]) && (m[2][1]==mt.m[2][1]) && (m[0][2]==mt.m[2][2]));
}

const MATRIX3X3 MATRIX3X3::Inverse(void)
{
	MATRIX3X3 r(1,0,0,0,1,0,0,0,1);
	float _m[4][4],tp;	memcpy(_m,m,4*4*sizeof(float));

	tp=_m[0][0];	_m[0][0]/=tp;			_m[0][1]/=tp;			_m[0][2]/=tp;			r.m[0][0]/=tp;
	tp=_m[1][0];	_m[1][0]-=tp*_m[0][0];	_m[1][1]-=tp*_m[0][1];	_m[1][2]-=tp*_m[0][2];	r.m[1][0]-=tp*r.m[0][0];
	tp=_m[2][0];	_m[2][0]-=tp*_m[0][0];	_m[2][1]-=tp*_m[0][1];	_m[2][2]-=tp*_m[0][2];	r.m[2][0]-=tp*r.m[0][0];

	tp=_m[1][1];	_m[1][1]/=tp;			_m[1][2]/=tp;			r.m[1][0]/=tp;				r.m[1][1]/=tp;
	tp=_m[0][1];	_m[0][1]-=tp*_m[1][1];	_m[0][2]-=tp*_m[1][2];	r.m[0][0]-=tp*r.m[1][0];	r.m[0][1]-=tp*r.m[1][1];
	tp=_m[2][1];	_m[2][1]-=tp*_m[1][1];	_m[2][2]-=tp*_m[1][2];	r.m[2][0]-=tp*r.m[1][0];	r.m[2][1]-=tp*r.m[1][1];

	tp=_m[2][2];	_m[2][2]/=tp;			r.m[2][0]/=tp;				r.m[2][1]/=tp;				r.m[2][2]/=tp;
	tp=_m[0][2];	_m[0][2]-=tp*_m[2][2];	r.m[0][0]-=tp*r.m[2][0];	r.m[0][1]-=tp*r.m[2][1];	r.m[0][2]-=tp*r.m[2][2];
	tp=_m[1][2];	_m[1][2]-=tp*_m[2][2];	r.m[1][0]-=tp*r.m[2][0];	r.m[1][1]-=tp*r.m[2][1];	r.m[1][2]-=tp*r.m[2][2];

	return r;
}

const MATRIX3X3 MATRIX3X3::Transpose(void)
{
	MATRIX3X3 r=*this;

	r.m[0][0]=m[0][0];	r.m[0][1]=m[1][0];	r.m[0][2]=m[2][0];
	r.m[1][0]=m[0][1];	r.m[1][1]=m[1][1];	r.m[1][2]=m[2][1];
	r.m[2][0]=m[0][2];	r.m[2][1]=m[1][2];	r.m[2][2]=m[2][2];

	return r;
}

const void MATRIX3X3::SetProperties(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i)
{
	m[0][0]=_a;	m[0][1]=_b;	m[0][2]=_c;
	m[1][0]=_d;	m[1][1]=_e;	m[1][2]=_f;
	m[2][0]=_g;	m[2][1]=_h;	m[2][2]=_i;
}

MATRIX3X3::~MATRIX3X3()
{
}
/* <-- MATRIX3X3 */

/* MATRIX4X4 --> */
MATRIX4X4::MATRIX4X4()
{
}

MATRIX4X4::MATRIX4X4(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i,float _j,float _k,float _l,float _m,float _n,float _o,float _p)
{
	m[0][0]=_a;	m[0][1]=_b;	m[0][2]=_c;	m[0][3]=_d;
	m[1][0]=_e;	m[1][1]=_f;	m[1][2]=_g;	m[1][3]=_h;
	m[2][0]=_i;	m[2][1]=_j;	m[2][2]=_k;	m[2][3]=_l;
	m[3][0]=_m;	m[3][1]=_n;	m[3][2]=_o;	m[3][3]=_p;
}

const MATRIX4X4 MATRIX4X4::operator=(const MATRIX4X4 &mt)
{
	m[0][0]=mt.m[0][0];	m[0][1]=mt.m[0][1];	m[0][2]=mt.m[0][2];	m[0][3]=mt.m[0][3];
	m[1][0]=mt.m[1][0];	m[1][1]=mt.m[1][1];	m[1][2]=mt.m[1][2];	m[1][3]=mt.m[1][3];
	m[2][0]=mt.m[2][0];	m[2][1]=mt.m[2][1];	m[2][2]=mt.m[2][2];	m[2][3]=mt.m[2][3];
	m[3][0]=mt.m[3][0];	m[3][1]=mt.m[3][1];	m[3][2]=mt.m[3][2];	m[3][3]=mt.m[3][3];
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator+(const MATRIX4X4 &mt)
{
	return MATRIX4X4(
	m[0][0]+mt.m[0][0],m[0][1]+mt.m[0][1],m[0][2]+mt.m[0][2],m[0][3]+mt.m[0][3],
	m[1][0]+mt.m[1][0],m[1][1]+mt.m[1][1],m[0][2]+mt.m[1][2],m[1][3]+mt.m[1][3],
	m[2][0]+mt.m[2][0],m[2][1]+mt.m[2][1],m[0][2]+mt.m[2][2],m[2][3]+mt.m[2][3],
	m[3][0]+mt.m[3][0],m[3][1]+mt.m[3][1],m[0][2]+mt.m[3][2],m[3][3]+mt.m[3][3]);
}

const MATRIX4X4 MATRIX4X4::operator+(void)
{
	return *this;
};

const MATRIX4X4 MATRIX4X4::operator+=(const MATRIX4X4 &mt)
{
	m[0][0]+=mt.m[0][0];	m[0][1]+=mt.m[0][1];	m[0][2]+=mt.m[0][2];	m[0][3]+=mt.m[0][3];
	m[1][0]+=mt.m[1][0];	m[1][1]+=mt.m[1][1];	m[0][2]+=mt.m[1][2];	m[1][3]+=mt.m[1][3];
	m[2][0]+=mt.m[2][0];	m[2][1]+=mt.m[2][1];	m[0][2]+=mt.m[2][2];	m[2][3]+=mt.m[2][3];
	m[3][0]+=mt.m[3][0];	m[3][1]+=mt.m[3][1];	m[0][2]+=mt.m[3][2];	m[3][3]+=mt.m[3][3];
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator-(const MATRIX4X4 &mt)
{
	return MATRIX4X4(
	m[0][0]-mt.m[0][0],m[0][1]-mt.m[0][1],m[0][2]-mt.m[0][2],m[0][3]-mt.m[0][3],
	m[1][0]-mt.m[1][0],m[1][1]-mt.m[1][1],m[0][2]-mt.m[1][2],m[1][3]-mt.m[1][3],
	m[2][0]-mt.m[2][0],m[2][1]-mt.m[2][1],m[0][2]-mt.m[2][2],m[2][3]-mt.m[2][3],
	m[3][0]-mt.m[3][0],m[3][1]-mt.m[3][1],m[0][2]-mt.m[3][2],m[3][3]-mt.m[3][3]);
}

const MATRIX4X4 MATRIX4X4::operator-(void)
{
	return MATRIX4X4(
	-m[0][0],-m[0][1],-m[0][2],-m[0][3],
	-m[1][0],-m[1][1],-m[0][2],-m[1][3],
	-m[2][0],-m[2][1],-m[0][2],-m[2][3],
	-m[3][0],-m[3][1],-m[0][2],-m[3][3]);
}

const MATRIX4X4 MATRIX4X4::operator-=(const MATRIX4X4 &mt)
{
	m[0][0]-=mt.m[0][0];	m[0][1]-=mt.m[0][1];	m[0][2]-=mt.m[0][2];	m[0][3]-=mt.m[0][3];
	m[1][0]-=mt.m[1][0];	m[1][1]-=mt.m[1][1];	m[0][2]-=mt.m[1][2];	m[1][3]-=mt.m[1][3];
	m[2][0]-=mt.m[2][0];	m[2][1]-=mt.m[2][1];	m[0][2]-=mt.m[2][2];	m[2][3]-=mt.m[2][3];
	m[3][0]-=mt.m[3][0];	m[3][1]-=mt.m[3][1];	m[0][2]-=mt.m[3][2];	m[3][3]-=mt.m[3][3];
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator*(const MATRIX4X4 &mt)
{
	return MATRIX4X4(
	m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0]+m[0][2]*mt.m[2][0]+m[0][3]*mt.m[3][0],
	m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1]+m[0][2]*mt.m[2][1]+m[0][3]*mt.m[3][1],
	m[0][0]*mt.m[0][2]+m[0][1]*mt.m[1][2]+m[0][2]*mt.m[2][2]+m[0][3]*mt.m[3][2],
	m[0][0]*mt.m[0][3]+m[0][1]*mt.m[1][3]+m[0][2]*mt.m[2][3]+m[0][3]*mt.m[3][3],
	m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0]+m[1][2]*mt.m[2][0]+m[1][3]*mt.m[3][0],
	m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1]+m[1][2]*mt.m[2][1]+m[1][3]*mt.m[3][1],
	m[1][0]*mt.m[0][2]+m[1][1]*mt.m[1][2]+m[1][2]*mt.m[2][2]+m[1][3]*mt.m[3][2],
	m[1][0]*mt.m[0][3]+m[1][1]*mt.m[1][3]+m[1][2]*mt.m[2][3]+m[1][3]*mt.m[3][3],
	m[2][0]*mt.m[0][0]+m[2][1]*mt.m[1][0]+m[2][2]*mt.m[2][0]+m[2][3]*mt.m[3][0],
	m[2][0]*mt.m[0][1]+m[2][1]*mt.m[1][1]+m[2][2]*mt.m[2][1]+m[2][3]*mt.m[3][1],
	m[2][0]*mt.m[0][2]+m[2][1]*mt.m[1][2]+m[2][2]*mt.m[2][2]+m[2][3]*mt.m[3][2],
	m[2][0]*mt.m[0][3]+m[2][1]*mt.m[1][3]+m[2][2]*mt.m[2][3]+m[2][3]*mt.m[3][3],
	m[3][0]*mt.m[0][0]+m[3][1]*mt.m[1][0]+m[3][2]*mt.m[2][0]+m[3][3]*mt.m[3][0],
	m[3][0]*mt.m[0][1]+m[3][1]*mt.m[1][1]+m[3][2]*mt.m[2][1]+m[3][3]*mt.m[3][1],
	m[3][0]*mt.m[0][2]+m[3][1]*mt.m[1][2]+m[3][2]*mt.m[2][2]+m[3][3]*mt.m[3][2],
	m[3][0]*mt.m[0][3]+m[3][1]*mt.m[1][3]+m[3][2]*mt.m[2][3]+m[3][3]*mt.m[3][3]);
}

const MATRIX4X4 MATRIX4X4::operator*(const float s)
{
	return MATRIX4X4(
	m[0][0]*s,m[0][1]*s,m[0][2]*s,m[0][3]*s,
	m[1][0]*s,m[1][1]*s,m[1][2]*s,m[1][3]*s,
	m[2][0]*s,m[2][1]*s,m[2][2]*s,m[2][3]*s,
	m[3][0]*s,m[3][1]*s,m[3][2]*s,m[3][3]*s);
}

const MATRIX4X4 MATRIX4X4::operator*=(const MATRIX4X4 &mt)
{
	m[0][0]=m[0][0]*mt.m[0][0]+m[0][1]*mt.m[1][0]+m[0][2]*mt.m[2][0]+m[0][3]*mt.m[3][0];
	m[0][1]=m[0][0]*mt.m[0][1]+m[0][1]*mt.m[1][1]+m[0][2]*mt.m[2][1]+m[0][3]*mt.m[3][1];
	m[0][2]=m[0][0]*mt.m[0][2]+m[0][1]*mt.m[1][2]+m[0][2]*mt.m[2][2]+m[0][3]*mt.m[3][2];
	m[0][3]=m[0][0]*mt.m[0][3]+m[0][1]*mt.m[1][3]+m[0][2]*mt.m[2][3]+m[0][3]*mt.m[3][3];
	m[1][0]=m[1][0]*mt.m[0][0]+m[1][1]*mt.m[1][0]+m[1][2]*mt.m[2][0]+m[1][3]*mt.m[3][0];
	m[1][1]=m[1][0]*mt.m[0][1]+m[1][1]*mt.m[1][1]+m[1][2]*mt.m[2][1]+m[1][3]*mt.m[3][1];
	m[1][2]=m[1][0]*mt.m[0][2]+m[1][1]*mt.m[1][2]+m[1][2]*mt.m[2][2]+m[1][3]*mt.m[3][2];
	m[1][3]=m[1][0]*mt.m[0][3]+m[1][1]*mt.m[1][3]+m[1][2]*mt.m[2][3]+m[1][3]*mt.m[3][3];
	m[2][0]=m[2][0]*mt.m[0][0]+m[2][1]*mt.m[1][0]+m[2][2]*mt.m[2][0]+m[2][3]*mt.m[3][0];
	m[2][1]=m[2][0]*mt.m[0][1]+m[2][1]*mt.m[1][1]+m[2][2]*mt.m[2][1]+m[2][3]*mt.m[3][1];
	m[2][2]=m[2][0]*mt.m[0][2]+m[2][1]*mt.m[1][2]+m[2][2]*mt.m[2][2]+m[2][3]*mt.m[3][2];
	m[2][3]=m[2][0]*mt.m[0][3]+m[2][1]*mt.m[1][3]+m[2][2]*mt.m[2][3]+m[2][3]*mt.m[3][3];
	m[3][0]=m[3][0]*mt.m[0][0]+m[3][1]*mt.m[1][0]+m[3][2]*mt.m[2][0]+m[3][3]*mt.m[3][0];
	m[3][1]=m[3][0]*mt.m[0][1]+m[3][1]*mt.m[1][1]+m[3][2]*mt.m[2][1]+m[3][3]*mt.m[3][1];
	m[3][2]=m[3][0]*mt.m[0][2]+m[3][1]*mt.m[1][2]+m[3][2]*mt.m[2][2]+m[3][3]*mt.m[3][2];
	m[3][3]=m[3][0]*mt.m[0][3]+m[3][1]*mt.m[1][3]+m[3][2]*mt.m[2][3]+m[3][3]*mt.m[3][3];
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator*=(const float s)
{
	m[0][0]*=s;	m[0][1]*=s;	m[0][2]*=s;	m[0][3]*=s;
	m[1][0]*=s;	m[1][1]*=s;	m[1][2]*=s;	m[1][3]*=s;
	m[2][0]*=s;	m[2][1]*=s;	m[2][2]*=s;	m[2][3]*=s;
	m[3][0]*=s;	m[3][1]*=s;	m[3][2]*=s;	m[3][3]*=s;
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator/(const MATRIX4X4 &mt)
{
//~	return *this*mt.Inverse();
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator/(const float s)
{
	return MATRIX4X4(
	m[0][0]/s,m[0][1]/s,m[0][2]/s,m[0][3]/s,
	m[1][0]/s,m[1][1]/s,m[1][2]/s,m[1][3]/s,
	m[2][0]/s,m[2][1]/s,m[2][2]/s,m[2][3]/s,
	m[3][0]/s,m[3][1]/s,m[3][2]/s,m[3][3]/s);
}

const MATRIX4X4 MATRIX4X4::operator/=(const MATRIX4X4 &mt)
{
//~	*this*=mt.Inverse();
	return *this;
}

const MATRIX4X4 MATRIX4X4::operator/=(const float s)
{
	m[0][0]/=s;	m[0][1]/=s;	m[0][2]/=s;	m[0][3]/=s;
	m[1][0]/=s;	m[1][1]/=s;	m[1][2]/=s;	m[1][3]/=s;
	m[2][0]/=s;	m[2][1]/=s;	m[2][2]/=s;	m[2][3]/=s;
	m[3][0]/=s;	m[3][1]/=s;	m[3][2]/=s;	m[3][3]/=s;
	return *this;
}

const bool MATRIX4X4::operator==(const MATRIX4X4 &mt)
{
	return (
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) && (m[0][2]==mt.m[0][2]) && (m[0][3]==mt.m[0][3]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]) && (m[1][2]==mt.m[1][2]) && (m[1][3]==mt.m[1][3]) &&
	(m[2][0]==mt.m[2][0]) && (m[2][1]==mt.m[2][1]) && (m[2][2]==mt.m[2][2]) && (m[2][3]==mt.m[2][3]) &&
	(m[3][0]==mt.m[3][0]) && (m[3][1]==mt.m[3][1]) && (m[3][2]==mt.m[3][2]) && (m[3][3]==mt.m[3][3]));
}

const bool MATRIX4X4::operator!=(const MATRIX4X4 &mt)
{
	return !(
	(m[0][0]==mt.m[0][0]) && (m[0][1]==mt.m[0][1]) && (m[0][2]==mt.m[0][2]) && (m[0][3]==mt.m[0][3]) &&
	(m[1][0]==mt.m[1][0]) && (m[1][1]==mt.m[1][1]) && (m[0][2]==mt.m[1][2]) && (m[1][3]==mt.m[1][3]) &&
	(m[2][0]==mt.m[2][0]) && (m[2][1]==mt.m[2][1]) && (m[0][2]==mt.m[2][2]) && (m[2][3]==mt.m[2][3]) &&
	(m[3][0]==mt.m[3][0]) && (m[3][1]==mt.m[3][1]) && (m[0][2]==mt.m[3][2]) && (m[3][3]==mt.m[3][3]));
}

const float MATRIX4X4::CoFactor(char row,char col)
{
	return ((m[(row+1)&3][(col+1)&3]*m[(row+2)&3][(col+2)&3]*m[(row+3)&3][(col+3)&3]+m[(row+1)&3][(col+2)&3]*m[(row+2)&3][(col+3)&3]*m[(row+3)&3][(col+1)&3]+m[(row+1)&3][(col+3)&3]*m[(row+2)&3][(col+1)&3]*m[(row+3)&3][(col+2)&3])-(m[(row+3)&3][(col+1)&3]*m[(row+2)&3][(col+2)&3]*m[(row+1)&3][(col+3)&3]+m[(row+3)&3][(col+2)&3]*m[(row+2)&3][(col+3)&3]*m[(row+1)&3][(col+1)&3]+m[(row+3)&3][(col+3)&3]*m[(row+2)&3][(col+1)&3]*m[(row+1)&3][(col+2)&3]))*((row+col) & 1 ? -1.0f : +1.0f);
}

const float MATRIX4X4::Determinant(void)
{
	return m[0][0]*CoFactor(0,0)+m[0][1]*CoFactor(0,1)+m[0][2]*CoFactor(0,2)+m[0][3]*CoFactor(0,3);
}

const MATRIX4X4 MATRIX4X4::CFInverse(void)
{
	if(byKabs(Determinant())<(1.0e-7f)){	return *this;	}
	MATRIX4X4 r;	float IDet=1.0f/Determinant();

	r.m[0][0]=CoFactor(0,0)*IDet;
	r.m[1][0]=CoFactor(0,1)*IDet;
	r.m[2][0]=CoFactor(0,2)*IDet;
	r.m[3][0]=CoFactor(0,3)*IDet;

	r.m[0][1]=CoFactor(1,0)*IDet;
	r.m[1][1]=CoFactor(1,1)*IDet;
	r.m[2][1]=CoFactor(1,2)*IDet;
	r.m[3][1]=CoFactor(1,3)*IDet;

	r.m[0][2]=CoFactor(2,0)*IDet;
	r.m[1][2]=CoFactor(2,1)*IDet;
	r.m[2][2]=CoFactor(2,2)*IDet;
	r.m[3][2]=CoFactor(2,3)*IDet;

	r.m[0][3]=CoFactor(3,0)*IDet;
	r.m[1][3]=CoFactor(3,1)*IDet;
	r.m[2][3]=CoFactor(3,2)*IDet;
	r.m[3][3]=CoFactor(3,3)*IDet;

	return r;
}

const MATRIX4X4 MATRIX4X4::Inverse(void)
{
	MATRIX4X4 r(1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1);
	float _m[4][4],tp;	memcpy(_m,m,4*4*sizeof(float));

	tp=_m[0][0];	_m[0][0]/=tp;			_m[0][1]/=tp;			_m[0][2]/=tp;			_m[0][3]/=tp;			r.m[0][0]/=tp;
	tp=_m[1][0];	_m[1][0]-=tp*_m[0][0];	_m[1][1]-=tp*_m[0][1];	_m[1][2]-=tp*_m[0][2];	_m[1][3]-=tp*_m[0][3];	r.m[1][0]-=tp*r.m[0][0];
	tp=_m[2][0];	_m[2][0]-=tp*_m[0][0];	_m[2][1]-=tp*_m[0][1];	_m[2][2]-=tp*_m[0][2];	_m[2][3]-=tp*_m[0][3];	r.m[2][0]-=tp*r.m[0][0];
	tp=_m[3][0];	_m[3][0]-=tp*_m[0][0];	_m[3][1]-=tp*_m[0][1];	_m[3][2]-=tp*_m[0][2];	_m[3][3]-=tp*_m[0][3];	r.m[3][0]-=tp*r.m[0][0];

	tp=_m[1][1];	_m[1][1]/=tp;			_m[1][2]/=tp;			_m[1][3]/=tp;			r.m[1][0]/=tp;				r.m[1][1]/=tp;
	tp=_m[0][1];	_m[0][1]-=tp*_m[1][1];	_m[0][2]-=tp*_m[1][2];	_m[0][3]-=tp*_m[1][3];	r.m[0][0]-=tp*r.m[1][0];	r.m[0][1]-=tp*r.m[1][1];
	tp=_m[2][1];	_m[2][1]-=tp*_m[1][1];	_m[2][2]-=tp*_m[1][2];	_m[2][3]-=tp*_m[1][3];	r.m[2][0]-=tp*r.m[1][0];	r.m[2][1]-=tp*r.m[1][1];
	tp=_m[3][1];	_m[3][1]-=tp*_m[1][1];	_m[3][2]-=tp*_m[1][2];	_m[3][3]-=tp*_m[1][3];	r.m[3][0]-=tp*r.m[1][0];	r.m[3][1]-=tp*r.m[1][1];

	tp=_m[2][2];	_m[2][2]/=tp;			_m[2][3]/=tp;			r.m[2][0]/=tp;				r.m[2][1]/=tp;				r.m[2][2]/=tp;
	tp=_m[0][2];	_m[0][2]-=tp*_m[2][2];	_m[0][3]-=tp*_m[2][3];	r.m[0][0]-=tp*r.m[2][0];	r.m[0][1]-=tp*r.m[2][1];	r.m[0][2]-=tp*r.m[2][2];
	tp=_m[1][2];	_m[1][2]-=tp*_m[2][2];	_m[1][3]-=tp*_m[2][3];	r.m[1][0]-=tp*r.m[2][0];	r.m[1][1]-=tp*r.m[2][1];	r.m[1][2]-=tp*r.m[2][2];
	tp=_m[3][2];	_m[3][2]-=tp*_m[2][2];	_m[3][3]-=tp*_m[2][3];	r.m[3][0]-=tp*r.m[2][0];	r.m[3][1]-=tp*r.m[2][1];	r.m[3][2]-=tp*r.m[2][2];

	tp=_m[3][3];	_m[3][3]/=tp;			r.m[3][0]/=tp;				r.m[3][1]/=tp;				r.m[3][2]/=tp;				r.m[3][3]/=tp;
	tp=_m[0][3];	_m[0][3]-=tp*_m[3][3];	r.m[0][0]-=tp*r.m[3][0];	r.m[0][1]-=tp*r.m[3][1];	r.m[0][2]-=tp*r.m[3][2];	r.m[0][3]-=tp*r.m[3][3];
	tp=_m[1][3];	_m[1][3]-=tp*_m[3][3];	r.m[1][0]-=tp*r.m[3][0];	r.m[1][1]-=tp*r.m[3][1];	r.m[1][2]-=tp*r.m[3][2];	r.m[1][3]-=tp*r.m[3][3];
	tp=_m[2][3]; 	_m[2][3]-=tp*_m[3][3];	r.m[2][0]-=tp*r.m[3][0];	r.m[2][1]-=tp*r.m[3][1];	r.m[2][2]-=tp*r.m[3][2];	r.m[2][3]-=tp*r.m[3][3];

	return r;
}

const MATRIX4X4 MATRIX4X4::PRInverse(void)
{
	MATRIX4X4 r;

	r.m[0][0]=m[0][0];	r.m[1][0]=m[0][1];	r.m[2][0]=m[0][2];	r.m[3][0]=-(m[3][0]*m[0][0]+m[3][1]*m[0][1]+m[3][2]*m[0][2]);
	r.m[0][1]=m[1][0];	r.m[1][1]=m[1][1];	r.m[2][1]=m[1][2];	r.m[3][1]=-(m[3][0]*m[1][0]+m[3][1]*m[1][1]+m[3][2]*m[1][2]);
	r.m[0][2]=m[2][0];	r.m[1][2]=m[2][1];	r.m[2][2]=m[2][2];	r.m[3][2]=-(m[3][0]*m[2][0]+m[3][1]*m[2][1]+m[3][2]*m[2][2]);
	r.m[0][3]=0.0f;		r.m[1][3]=0.0f;		r.m[2][3]=0.0f;		r.m[3][3]=1.0f;

	return r;
}

const MATRIX4X4 MATRIX4X4::Transpose(void)
{
	MATRIX4X4 r=*this;

	r.m[0][0]=m[0][0];	r.m[0][1]=m[1][0];	r.m[0][2]=m[2][0];	r.m[0][3]=m[3][0];
	r.m[1][0]=m[0][1];	r.m[1][1]=m[1][1];	r.m[1][2]=m[2][1];	r.m[1][3]=m[3][1];
	r.m[2][0]=m[0][2];	r.m[2][1]=m[1][2];	r.m[2][2]=m[2][2];	r.m[2][3]=m[3][2];
	r.m[3][0]=m[0][3];	r.m[3][1]=m[1][3];	r.m[3][2]=m[2][3];	r.m[3][3]=m[3][3];

	return r;
}

const POINT3 MATRIX4X4::Point(const POINT3 &p)
{
	POINT4 q;	POINT3 r;
	
	q.x=p.x*m[0][0]+p.y*m[1][0]+p.z*m[2][0]+m[3][0];
	q.y=p.x*m[0][1]+p.y*m[1][1]+p.z*m[2][1]+m[3][1];
	q.z=p.x*m[0][2]+p.y*m[1][2]+p.z*m[2][2]+m[3][2];
	q.w=p.x*m[0][3]+p.y*m[1][3]+p.z*m[2][3]+m[3][3];

	r.x=q.x/q.w;	r.y=q.y/q.w;	r.z=q.z/q.w;
	return r;
}

const POINT4 MATRIX4X4::Point(const POINT4 &p)
{
	POINT4 q;
	
	q.x=p.x*m[0][0]+p.y*m[1][0]+p.z*m[2][0]+p.w*m[3][0];
	q.y=p.x*m[0][1]+p.y*m[1][1]+p.z*m[2][1]+p.w*m[3][1];
	q.z=p.x*m[0][2]+p.y*m[1][2]+p.z*m[2][2]+p.w*m[3][2];
	q.w=p.x*m[0][3]+p.y*m[1][3]+p.z*m[2][3]+p.w*m[3][3];

	return q;    
}

const void MATRIX4X4::SetProperties(float _a,float _b,float _c,float _d,float _e,float _f,float _g,float _h,float _i,float _j,float _k,float _l,float _m,float _n,float _o,float _p)
{
	m[0][0]=_a;	m[0][1]=_b;	m[0][2]=_c;	m[0][3]=_d;
	m[1][0]=_e;	m[1][1]=_f;	m[1][2]=_g;	m[1][3]=_h;
	m[2][0]=_i;	m[2][1]=_j;	m[2][2]=_k;	m[2][3]=_l;
	m[3][0]=_m;	m[3][1]=_n;	m[3][2]=_o;	m[3][3]=_p;
}

MATRIX4X4::~MATRIX4X4()
{
}
/* <-- MATRIX4X4 */
