/* FILE: model.c
 *		Reads animated model files and renders them
 *
 * Monstrous Software (c) 2000-2001
 * http://home.planet.nl/~monstrousn
 *
 * MS3D file format loading based on MSViewer by Mete Ciragan, MilkShape3D
 *
 * Defines classes: Vec, Shape, Material, Bone and Model
 */

#include "modelosMS3D.h"

// Vector x Matrix (based on Brett Porter's Vector class)
//
void Vec::transform( const Matrix& m )
{
  double vector[4];
  const float *matrix = m.getMatrix();

  vector[0] = x*matrix[0]+y*matrix[4]+z*matrix[8]+matrix[12];
  vector[1] = x*matrix[1]+y*matrix[5]+z*matrix[9]+matrix[13];
  vector[2] = x*matrix[2]+y*matrix[6]+z*matrix[10]+matrix[14];
  vector[3] = x*matrix[3]+y*matrix[7]+z*matrix[11]+matrix[15];

  x = ( float )( vector[0] );
  y = ( float )( vector[1] );
  z = ( float )( vector[2] );
  w = ( float )( vector[3] );
}

void Vec::transform3( const Matrix& m )
{
  double vector[3];
  const float *matrix = m.getMatrix();

  vector[0] = x*matrix[0]+y*matrix[4]+z*matrix[8];
  vector[1] = x*matrix[1]+y*matrix[5]+z*matrix[9];
  vector[2] = x*matrix[2]+y*matrix[6]+z*matrix[10];

  x = ( float )( vector[0] );
  y = ( float )( vector[1] );
  z = ( float )( vector[2] );
}

/*
 * Shape 
 */


Shape::Shape()
{
  num_vertices = 0;
  vertices = NULL;
  num_triangles = 0;
  triangles = NULL;
  num_normals = 0;
  normals = NULL;
  Radio=0;
  Alto=0;

  MayaIA = new bool[TamMayaIA*TamMayaIA];

  for (int i=0; i<TamMayaIA*TamMayaIA ;i++)
    {
      MayaIA[i] = false;
    }

}

Shape::~Shape()
{
  if(vertices != NULL)
    delete[] vertices;	
  if(triangles != NULL)
    delete[] triangles;
  if(normals != NULL)
    delete[] normals;
  
  delete[] MayaIA;
}

  
//----------------------------------------------------------------------
void Shape::GetMaxMin(float *MaxMin, Bone *bones, float Escala)
{


	// BOUNDING BOXES

	MaxMin[0] = -100; // X Max
	MaxMin[1] = 100;  // X Min
	MaxMin[2] = -100; // Y Max
	MaxMin[3] = 100;  // Y Min
	MaxMin[4] = -100; // Z Max
	MaxMin[5] = 100;  // Z Min

  int	    i, j;
  Tri	    *tri;
  Vec	    *vec;
  Bone      *bone;
  float	    v[3];
  Matrix    *matrix;
  float Factor = 0.004*Escala;


  for(i = 0; i < num_triangles; i++)	// for each triangle
    {
      tri = triangles + i;				// pointer to triangle
      for(j = 2; j >= 0; j--)	// 3 vertices of the triangle
	{
	  
	  vec = vertices + tri->v[j];		// pointer to vertex
	  
	  if(bones == NULL)
	    {
	      if (vec->x*(Factor) > MaxMin[0]) MaxMin[0] = vec->x*(Factor);
	      else if (vec->x*(Factor) < MaxMin[1])	MaxMin[1] = vec->x*(Factor);
	      // --
	      if (vec->y*(Factor) > MaxMin[2]) MaxMin[2] = vec->y*(Factor);
	      else if (vec->y*(Factor) < MaxMin[3])	MaxMin[3] = vec->y*(Factor);
	      // --
	      if (vec->z*(Factor) > MaxMin[4]) MaxMin[4] = vec->z*(Factor);
	      else if (vec->z*(Factor) < MaxMin[5])	MaxMin[5] = vec->z*(Factor);
	      
	    }
	  else
	    {
	      bone = &bones[vec->bone];
	      matrix = & bone->m_final;
	      v[0] = vec->x;
	      v[1] = vec->y;
	      v[2] = vec->z;
	      matrix->rotateVect( v );
	      matrix->translateVect( v );

	      if (v[0]*(Factor) > MaxMin[0]) MaxMin[0] = v[0]*(Factor);
	      else if (v[0]*(Factor) < MaxMin[1])	MaxMin[1] = v[0]*(Factor);
	      // --
	      if (v[1]*(Factor) > MaxMin[2]) MaxMin[2] = v[1]*(Factor);
	      else if (v[1]*(Factor) < MaxMin[3])	MaxMin[3] = v[1]*(Factor);
	      // --
	      if (v[2]*(Factor) > MaxMin[4]) MaxMin[4] = v[2]*(Factor);
	      else if (v[2]*(Factor) < MaxMin[5])	MaxMin[5] = v[2]*(Factor);
	
	    }	
	 	
	}
    }
 Alto = MaxMin[2];

}
//------------------------------------------------
bool Shape::dibujame(Bone *bones, float Escala)
{
  
  int	    i, j;
  Tri	    *tri;
  Vec	    *vec;
  z3DVector *N;
  Bone      *bone;
  float	    v[3];
  Matrix    *matrix;

  for(i = 0; i < num_triangles; i++)	// for each triangle
    {
      tri = triangles + i;				// pointer to triangle
      for(j = 2; j >= 0; j--)	// 3 vertices of the triangle
	{
	  N = normals + tri->n[j];
	  glNormal3f(N->x, N->y, N->z);	// set normal vector  (object space)
	  vec = vertices + tri->v[j];		// pointer to vertex
	  glTexCoord2f (vec->u, vec->v);			// texture coordinate
	  
	  if(bones == NULL)
	    {
	      glVertex3f( vec->x*(0.004*Escala), 
			  vec->y*(0.004*Escala), 
			  vec->z*(0.004*Escala));
	    }
	  else
	    {


	      bone = &bones[vec->bone];
	      matrix = &bone->m_final;
	      v[0] = vec->x;
	      v[1] = vec->y;
	      v[2] = vec->z;

	      matrix->rotateVect( v );
	      matrix->translateVect( v );

	      v[0] = v[0]*(0.004*Escala);
	      v[1] = v[1]*(0.004*Escala);
	      v[2] = v[2]*(0.004*Escala);
	      glVertex3fv( v );	
	    }		
	}
    }


 return true;
}



bool Shape::loadFromMs3dAsciiSegment( FILE *file, float Escala)

{

  char szLine[256];
  int nFlags, nIndex, j;

  //
  // vertices
  //
  if (!fgets (szLine, 256, file))
    {
      return false;
    }
    
  if (sscanf (szLine, "%d", &num_vertices) != 1)
    {
      return false;
    }
  //cout << "Numero de vertices:" << num_vertices << endl;
  vertices = new Vec[num_vertices];
  for (j = 0; j < num_vertices; j++)
    {

      if (!fgets (szLine, 256, file))
	{
	  return false;
	}

      if (sscanf (szLine, "%d %f %f %f %f %f %d",
		  &nFlags,
		  &vertices[j].x, &vertices[j].y, &vertices[j].z,
		  &vertices[j].u, &vertices[j].v,
		  &vertices[j].bone
		  ) != 7)
	{
	  return false;
	}

      // adjust the y direction of the texture coordinate

      vertices[j].v = 1.0f - vertices[j].v;

 // Obtenemos el Radio
      if (vertices[j].y < 12.0)
	{
	  if (fabs(vertices[j].x) > Radio)
	    {
	      Radio = fabs(vertices[j].x);
	    }
	  if (fabs(vertices[j].z) > Radio)
	    { 
	      Radio = fabs(vertices[j].z);
	    }

	  MayaIA[(TamMayaIA/2 + (int)(4 * Densidad * (vertices[j].x * 0.004 * Escala))) * TamMayaIA + 
		 (TamMayaIA/2 + (int)(4 * Densidad * (vertices[j].z * 0.004 * Escala)))] = true;

	}
    }

  /*  
  int x,y;
  for (y=0 ; y<TamMayaIA ; y++)
    {
      for (x=0 ; x<TamMayaIA ; x++)
	{
	  if (MayaIA[y*TamMayaIA + x])
	    { cout << " 1"; }
	  else
	    { cout << " 0"; }
	}
      cout << endl;
    }
  */
  //
  // normals
  //
  if (!fgets (szLine, 256, file))
    {
      return false;
    }

  if (sscanf (szLine, "%d", &num_normals) != 1)
    {
      return false;
    }

		
  //cout << "Numero de normales:" << num_normals << endl;
  normals = new z3DVector[num_normals];

  for (j = 0; j < num_normals; j++)
    {
      if (!fgets (szLine, 256, file))
	{

	  return false;

	}	

      if (sscanf (szLine, "%f %f %f", 
		  &normals[j].x, &normals[j].y, &normals[j].z) != 3)
	{	
	  return false;
	}
				
      //cout << j << " Normal: " << normals[j].x << endl;

    }

  //

  // triangles

  //

  if (!fgets (szLine, 256, file))

    {

      return false;

    }

    

  if (sscanf (szLine, "%d", &num_triangles) != 1)
    {
      return false;
    }
  //cout << "Numero de triangulo:" << num_triangles << endl;
  triangles = new Tri[num_triangles];

  for (j = 0; j < num_triangles; j++)
    {
      if (!fgets (szLine, 256, file))
	{
	  return false;
	}
      if (sscanf (szLine, "%d %d %d %d %d %d %d %d",
		  &nFlags,
		  &triangles[j].v[0], &triangles[j].v[1], &triangles[j].v[2],
		  &triangles[j].n[0], &triangles[j].n[1], &triangles[j].n[2],
		  &nIndex
		  ) != 8)
	{
	  return false;
	}
      //cout << j << " Triangulos: " << triangles[j].v[0] << endl;
    }
  return true;
}


//-------------------------------
/// BONES///
//-------------------------------


Bone::Bone()	

{

}



Bone::~Bone()	
{
  if(PositionKeyFrames != NULL)
    {
      delete[] PositionKeyFrames;
      PositionKeyFrames = NULL;
    }

  if(RotationKeyFrames != NULL)
    {
      delete[] RotationKeyFrames;
      RotationKeyFrames = NULL;
    }
}


bool Bone::loadFromMs3dAsciiSegment( FILE *file )

{

  char szLine[256];
  int	 j;
  int	 nFlags;

  // name
  if (!fgets (szLine, 256, file))
    return false;

  if (sscanf (szLine, "\"%[^\"]\"", Name) != 1)
    return false;

  // parent name
  if (!fgets (szLine, 256, file))
    return false;
  strcpy (ParentName, "");
  sscanf (szLine, "\"%[^\"]\"", ParentName);

  if (!fgets (szLine, 256, file))
    return false;
  if (sscanf (szLine, "%d %f %f %f %f %f %f",
	      &nFlags,
	      &startPosition[0], &startPosition[1], &startPosition[2],
	      &startRotation[0], &startRotation[1], &startRotation[2]) != 7)
    return false;


 
  // position key count
  if (!fgets (szLine, 256, file))
    return false;
  if (sscanf (szLine, "%d", &NumPositionKeys) != 1)
    return false;
  //cout << "Posiciones: " << NumPositionKeys << endl;
  PositionKeyFrames = new KeyFrame[NumPositionKeys];
  for (j = 0; j < NumPositionKeys; j++)
    {
      if (!fgets (szLine, 256, file))
	return false;
      if (sscanf (szLine, "%f %f %f %f", &PositionKeyFrames[j].Time,
		  &PositionKeyFrames[j].Value[0],
		  &PositionKeyFrames[j].Value[1],
		  &PositionKeyFrames[j].Value[2] ) != 4)
	return false;


    }


  // rotation key count
  if (!fgets (szLine, 256, file))
    return false;
  if (sscanf (szLine, "%d", &NumRotationKeys) != 1)
    return false;

  RotationKeyFrames = new KeyFrame[NumRotationKeys];
  for (j = 0; j < NumRotationKeys; j++)
    {
      if (!fgets (szLine, 256, file))
	return false;
      if (sscanf (szLine, "%f %f %f %f", &RotationKeyFrames[j].Time,
		  &RotationKeyFrames[j].Value[0],
		  &RotationKeyFrames[j].Value[1],
		  &RotationKeyFrames[j].Value[2] ) != 4)
	return false;
    }
	
  return true;
}


void Bone::initialize( void )

{
  Matrix m_rel;

  // Create a transformation matrix from the position and rotation 
  m_rel.setRotationRadians( startRotation );
  m_rel.setTranslation( startPosition );

  // Each bone's final matrix is its relative matrix concatenated onto its 
  // parent's final matrix (which in turn is ....)
  //
  if ( Parent == NULL )
    m_final.set( m_rel.getMatrix());
  else
    {
      m_final.set( Parent->m_final.getMatrix());
      m_final.postMultiply( m_rel );
    }
}


// ----------------------------------------
void Bone::advanceTo( float CurrentTime )
{
  int		i;
  float	deltaTime;
  float	fraction;
  float	Position[3];
  float	Rotation[3];



  /* 
   * Position
   */

  // Find appropriate position key frame
  i = 0;
  while( i < NumPositionKeys-1 && PositionKeyFrames[i].Time < CurrentTime )
    i++;


  if( i > 0 && i < NumPositionKeys-1 )
    {
      // Interpolate between 2 key frames

      // time between the 2 key frames
      deltaTime = PositionKeyFrames[i].Time - PositionKeyFrames[i-1].Time;


      // relative position of interpolation point to the keyframes [0..1]
      fraction = (CurrentTime - PositionKeyFrames[i-1].Time) / deltaTime;


      Position[0] = PositionKeyFrames[i-1].Value[0] + fraction * (PositionKeyFrames[i].Value[0] - PositionKeyFrames[i-1].Value[0]);
      Position[1] = PositionKeyFrames[i-1].Value[1] + fraction * (PositionKeyFrames[i].Value[1] - PositionKeyFrames[i-1].Value[1]);
      Position[2] = PositionKeyFrames[i-1].Value[2] + fraction * (PositionKeyFrames[i].Value[2] - PositionKeyFrames[i-1].Value[2]);
    }
  else
    {
      memcpy(Position, PositionKeyFrames[i].Value, 3*sizeof(float) );
    }

  /* 
   * Rotation
   */

  // Find appropriate rotation key frame
  i = 0;
  while( i < NumRotationKeys-1 && RotationKeyFrames[i].Time < CurrentTime )
    i++;

  if( i > 0 && i < NumRotationKeys-1 )
    {
      // Interpolate between 2 key frames

      // time between the 2 key frames
      deltaTime = RotationKeyFrames[i].Time - RotationKeyFrames[i-1].Time;

      // relative position of interpolation point to the keyframes [0..1]
      fraction = (CurrentTime - RotationKeyFrames[i-1].Time) / deltaTime;

      Rotation[0] = RotationKeyFrames[i-1].Value[0] + fraction * (RotationKeyFrames[i].Value[0] - RotationKeyFrames[i-1].Value[0]);
      Rotation[1] = RotationKeyFrames[i-1].Value[1] + fraction * (RotationKeyFrames[i].Value[1] - RotationKeyFrames[i-1].Value[1]);
      Rotation[2] = RotationKeyFrames[i-1].Value[2] + fraction * (RotationKeyFrames[i].Value[2] - RotationKeyFrames[i-1].Value[2]);
    }
  else
    {
      memcpy(Rotation, RotationKeyFrames[i].Value, 3*sizeof(float) );
    }

  // Now we know the position and rotation for this animation frame.
  // Let's calculate the transformation matrix (m_final) for this bone...

  Matrix m_rel;
  Matrix m_frame;

  // Create a transformation matrix from the position and rotation of this
  // joint in the rest position
  m_rel.setRotationRadians( startRotation );
  m_rel.setTranslation( startPosition );

  // Create a transformation matrix from the position and rotation 
  // m_frame: additional transformation for this frame of the animation 
  m_frame.setRotationRadians( Rotation );
  m_frame.setTranslation( Position );

  // Add the animation state to the rest position
  m_rel.postMultiply( m_frame );

  // 
  if ( Parent == NULL )					// this is the root node
    m_final.set( m_rel.getMatrix());	// m_final := m_rel
  else									// not the root node
    {
      // m_final := parent's m_final * m_rel (matrix concatenation)

      m_final.set( Parent->m_final.getMatrix());
      m_final.postMultiply( m_rel );
    }
}

/*
 * Model 
 */

// ----------------------------------------

bool Model::Mallado = false;

// ----------------------------------------
Model::Model()

{
  num_bones = 0;
  bones = NULL;
  numArmas= 0;
  NumAnimaciones = 0;
}

// ----------------------------------------
Model::~Model()
{
  if (bones != NULL)
    {
      delete[] bones;
      bones = NULL;
    }
}


// ----------------------------------------
Animacion  Model::getFrames(string anim)
{

  int i;

  //cout << "Anim: " << anim << endl;

  for (i=0 ; i<NumAnimaciones ; i++)
    {
      if (Animaciones[i].Nombre == anim)
	{
	  return Animaciones[i];
	}
    }

  Animacion IniFinAnim;

  War3DWarning("MODEL","No se encontro la animacion solicitada");
  War3DWarning("MODEL",  string("Animacion: [") + anim + "] en el archivo [" + AnimacionNombre +  "] no encontrada consulta con tu grafista favorito");

  IniFinAnim.Nombre = anim;
  IniFinAnim.Inicio = -1;
  IniFinAnim.Fin = -1;

  return IniFinAnim;
}


// ----------------------------------------
bool Model::loadAnimaciones(FILE *file)
{
  char szLine[256];
  int j;
  char Name [256];

  // Numero de Animaciones
  if (!fgets (szLine, 256, file))
    return false;

  if (sscanf (szLine, "Secuencias: %i", &NumAnimaciones) != 1)
    return false;

  //  cout << "Secuencias: " << NumAnimaciones  << endl;

  Animaciones = new Animacion[NumAnimaciones];


  for (j=0; j<NumAnimaciones ; j++)
    {
      if (!fgets (szLine, 256, file))
	return false;

      if (sscanf (szLine, "%s %i %i", Name,
		  &Animaciones[j].Inicio,
		  &Animaciones[j].Fin) == 3)
	{
	  
	  Animaciones[j].Nombre = Name;
	  /*
	    cout << Animaciones[j].Nombre << "  " << 
	    Animaciones[j].Inicio << "  " << 
	    Animaciones[j].Fin << endl;
	  */
	}
      else
	{
	  return false;
	}
    }
  return true;
}


// ----------------------------------------
bool Model::Inicia(string Modelo, string strTextura, string Arma01, string TexArma01, string Animacion, float Escala)
{
  

  ArmaNombre = Arma01;



  bool	bError = false;
  char	szLine[256];
  War3DAccion("MODELOS","Abriendo Arma 01: " + Arma01);

  Arma01 = Arma01 + ".mes";
  
  FILE *file = fopen (Arma01.c_str(), "rt");
  if (!file)
    {
      cout << "[[Error]] al abrir el Arma " << Arma01 << endl;
      return false;
    }
  
  fgets (szLine, 256, file);
  if (strlen(szLine) >= 1)
    {
      // cout << "ARMA: Comentarios: "  << szLine;
    }
  
  if( !modArma.loadFromMs3dAsciiSegment(file, Escala))
    {
      cout << "[[Error]] al abrir el Arma: " << Arma01 << endl;
      bError = true;
      return false;
    }

  fclose (file);

  numArmas++;
  Inicia(Modelo, strTextura, Animacion, Escala);




  // cout << "|TEX| Abriendo textura: " << TexArma01 << endl;
  TexArma01 = TexArma01 + ".tga";
  texArma = ServidorDeTexturas::ME()->Nueva(TexArma01.c_str());


  return (true);

}


// ----------------------------------------
bool Model::Inicia(string Modelo, string strTextura, string Animacion, float Escala)
{

  AnimacionNombre = Animacion;
  ModeloNombre = Modelo;

  bool	bError = false;
  char	szLine[256];
  this->Escala = Escala;
	
  War3DAccion("MODELOS", "Abriendo modelo: " + Modelo);

  Modelo = Modelo + ".mes";

  FILE *file = fopen (Modelo.c_str(), "rt");
  if (!file)
    {
      cout << "[[Error]] al abrir el archivo " << Modelo << endl;
      return false;
    }

  fgets (szLine, 256, file);
  if (strlen(szLine) >= 1)
    {
      // cout << "MES Comentarios: "  << szLine;
    }
  
  if( !modBicho.loadFromMs3dAsciiSegment(file, Escala))
    {
      cout << "[[Error]] al abrir el Modelo: " << Modelo << endl;
      bError = true;
      return false;
    }

  fclose (file);

  //continue;
  
  // Animacion 
  if (Animacion.length() > 0)
    {
      War3DAccion("MODELOS", "Abriendo animacion: " + Animacion);

      Animacion = Animacion + ".ani";

      file = fopen (Animacion.c_str(), "rt");
      if (!file)
	{
	  cout << "[[Error]] al abrir el archivo " << Animacion << endl;
	  return false;
	}

      fgets (szLine, 256, file);
      if (strlen(szLine) >= 1)
	{
	  //cout << "ANI Comentarios: "  << szLine;
	}

      fgets (szLine, 256, file);
      if (sscanf (szLine, "Frames: %f", &MaxTime) == 1)
	{
	  //cout << "Frames: "  << MaxTime << endl;
	}
      else
	{
	  cout << "[[Error]] al leer la animacion." << endl;
	  cout << "[[War3D .ANI format]] La segunda lineas del archivo .ani, deber tener la forma \"Frames: Numero\"." << endl;
	  return false;
	}

      loadAnimaciones(file);

      fgets (szLine, 256, file);
      if (sscanf (szLine, "Bones: %d", &num_bones) == 1)
	{
	  int i;
	  // cout << "Numero de bones:" << num_bones << endl;
	  bones = new Bone[num_bones];
				
	  for (i = 0; i < num_bones && !bError; i++)
	    {
	      if( !bones[i].loadFromMs3dAsciiSegment(file) )
		{
		  bError = true;
		  cout << "[ERROR] al leer los Bones" << endl;
		  break;
		}
	    }
	}
		  
      fclose (file);

    }

	  
  if(bError)
    {
      cout <<  "[[Error]] al abrir el modelo." << endl;
      return false;
    }

  if (Animacion.length() > 0)
    {
      if( !linkBones() )
	{
	  cout <<  "[[Error]] al linkear los bones" << endl;
	  return false;
	}	
    }
	

  // cout << "||TEX|| Abriendo textura: " << Textura << endl;
  strTextura = strTextura + ".tga";
  texBicho = ServidorDeTexturas::ME()->Nueva(strTextura.c_str());

  if (Animacion.length() > 0)
    {
      initializeBones();
      attachSkin(modBicho);
      if (numArmas == 1)
	{
	  attachSkin(modArma);
	}
    }

	// Obtener los Maximos y minimos DEL PRimer FRame para poder 
	// dibuar la caja de seleccion.
	for (int i = 0; i < num_bones; i++)           
    {
      bones[i].advanceTo( 1.0 );
    }
	modBicho.GetMaxMin(MaxMin, bones, Escala);

  return true;
}


// ---------------------------------
bool Model::linkBones( void )

{
  // The relationship between child and parent bones are defined
  // by each child bone providing the name of its parent bone.
  // This function builds corresponding pointers for faster lookup.
  //
  int i, j;

  // Link children to parent
  //

  for (i = 0; i < num_bones; i++)            
    {
      bones[i].Parent = NULL;

      if( strlen(bones[i].ParentName) >  0 )	// does bone have parent?
	{
	  for (j = 0; j < num_bones; j++)	// search for parent
	    {
	      if( strcmp(bones[j].Name, bones[i].ParentName) == 0)	// j is parent of i
		{
		  bones[i].Parent = &bones[j];
		  break;
		}
	    }
	  if ( bones[i].Parent == NULL) // Unable to find parent bone 
	    return false;
        }
    }
	
  return true;
}

void Model::initializeBones( void )
{
  int i;

  for (i = 0; i < num_bones; i++)            
    {
      bones[i].initialize();
    }
}


// TODO: Uses private members from various classes (temporarily made public). Clean up!
//

void Model::attachSkin(Shape &mod)
{
  int	j;
  int		bone;
  Matrix	*matrix;
  float	v[3];

  // Make vertex vector relative to the bone it's attached to
  for(j = 0; j < mod.num_vertices; j++ )	// for all vertices
    {
      bone = mod.vertices[j].bone;

      if( bone != -1 )	// if vertex attached to a bone
	{
	  matrix = & bones[bone].m_final;

	  // make relative to bone position and orientation
	  // modifies vertex x,y,z 

	  v[0] = mod.vertices[j].x;
	  v[1] = mod.vertices[j].y;
	  v[2] = mod.vertices[j].z;
	  matrix->inverseTranslateVect( v );
	  matrix->inverseRotateVect( v );
	  mod.vertices[j].x = v[0];
	  mod.vertices[j].y = v[1];
	  mod.vertices[j].z = v[2];
	}
    }
}


void Model::dibujame( float x, float y, float z, float frame, float angulo, float armaAngulo)
{

  for (int i = 0; i < num_bones; i++)           
    {
      if (string(bones[i].Name) == "GIRAN") 
	{
	  bones[i].startRotation[1] = (armaAngulo * 3.14) /180.0;
	}

      bones[i].advanceTo( frame );
    }

  glPushMatrix();
  glTranslatef(x, y, z);
  glRotatef(angulo, 0.0, 1.0, 0.0);


  //glColor3f(1.0, 1.0, 1.0);  // white

  // glEnable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);


  if (Model::Mallado) 
    {
      glDisable(GL_TEXTURE_2D);
      glLineWidth(1.0);
    }


  
  texBicho.usaGLTextura(TextAltaCalidad);
  if (Model::Mallado){ glBegin(GL_LINE_STRIP); }
  else glBegin(GL_TRIANGLES);
  {
    modBicho.dibujame(bones, Escala);
  }
  glEnd();
  

 if (numArmas == 1)
    {
      texArma.usaGLTextura(TextAltaCalidad); 
      if (Model::Mallado){ glBegin(GL_LINE_STRIP); }
      else glBegin(GL_TRIANGLES);
      {
	modArma.dibujame(bones, Escala);
      }
      glEnd();
    }


  glPopMatrix();	
}


//----------------------------------------------------------------------
void Model::dibujame( float x, float y, float z, float frame, float angulo, z3DVector Normal, float armaAngulo)
{


  for (int i = 0; i < num_bones; i++)           
    {

      if (string(bones[i].Name) == "GIRAN") 
	{
	  bones[i].startRotation[1] = (armaAngulo * 3.14) /180.0;
	}

      bones[i].advanceTo( frame );
    }


  float tmpx, tmpy;
  
  tmpx = -Normal.y*zsin(angulo) + Normal.x*zcos(angulo);  
  tmpy = -Normal.y*zcos(angulo) - Normal.x*zsin(angulo);
  
  //calculo de angulos: 

  float aX = zangulo(tmpx, Normal.z); 
  float aY = zangulo(tmpy, Normal.z); 

  glPushMatrix();

  glTranslatef(x, y, z);
  
  glRotatef(angulo,0,1,0); 
  glRotatef(aX,0,0,-1); 
  glRotatef(aY,-1,0,0); 


  glEnable(GL_TEXTURE_2D);

  if (Model::Mallado) 
    {
      glDisable(GL_TEXTURE_2D);
      glLineWidth(1.0);
    }


  
  texBicho.usaGLTextura(TextAltaCalidad);
  if (Model::Mallado){ glBegin(GL_LINE_STRIP); }
  else glBegin(GL_TRIANGLES);
  {
    modBicho.dibujame(bones, Escala);
  }
  glEnd();
  

 if (numArmas == 1)
    {
      texArma.usaGLTextura(TextAltaCalidad); 
      if (Model::Mallado){ glBegin(GL_LINE_STRIP); }
      else glBegin(GL_TRIANGLES);
      {
	modArma.dibujame(bones, Escala);
      }
      glEnd();
    }


  glPopMatrix();
  
  if(glGetError()!=0){	cout << "modelMS3D OPENGL: " << glGetError() << endl;}	
}


//----------------------------------------------------------------------
void Model::dibujameSelect( float x, float y, float z, float frame, float angulo)
{

  // BOUNDING BOXES

  glPushMatrix();
  glTranslatef(x, y, z);
  glRotatef(angulo, 0.0, 1.0, 0.0);


  glColor4f(1.0, 1.0, 1.0, 0.3);  // white

  glDisable(GL_TEXTURE_2D);
  glDisable(GL_LIGHTING);

  glBegin(GL_QUADS);
  {
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[4]);
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[4]);
    glVertex3f(MaxMin[0], MaxMin[3], MaxMin[4]);
    glVertex3f(MaxMin[1], MaxMin[3], MaxMin[4]);
    
    
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[4]);
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[0], MaxMin[3], MaxMin[5]);
    glVertex3f(MaxMin[0], MaxMin[3], MaxMin[4]);

    
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[4]);
    glVertex3f(MaxMin[1], MaxMin[3], MaxMin[4]);
    glVertex3f(MaxMin[1], MaxMin[3], MaxMin[5]);
    
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[1], MaxMin[3], MaxMin[5]);
    glVertex3f(MaxMin[0], MaxMin[3], MaxMin[5]);
    
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[5]);
    glVertex3f(MaxMin[0], MaxMin[2], MaxMin[4]);
    glVertex3f(MaxMin[1], MaxMin[2], MaxMin[4]);

  }
  glEnd();


  glPopMatrix();	
}






